; This file is part of RealSports Curling.

; RealSports Curling is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; RealSports Curling is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with RealSports Curling.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2018 Average Software.

; Game configuration routines

; Menu state identifiers.
menu_configure_game equ     0
menu_select_team_1  equ     1
menu_create_team_1  equ     2
menu_select_team_2  equ     3
menu_create_team_2  equ     4

; Menu data.
title_blink_timer   equ     $20
title_rock_timer    equ     $21
title_rock_pos      equ     $22
menu_type           equ     $23
active_team         equ     $24
active_player       equ     $25
coin_toss_winner    equ     $26
coin_low_byte       equ     $27
coin_high_byte      equ     $28
coin_timer          equ     $29
coin_countdown      equ     $2a

; Key debounce timer.
key_debounce        equ     $750

; Game configuration data.
game_length         equ     $751
game_type           equ     $752
curler_handedness   equ     $753
curler_jacket_color equ     curler_handedness + 8
curler_hair_color   equ     curler_jacket_color + 8
curler_hair_style   equ     curler_hair_color + 8
curler_pants_color  equ     curler_hair_style + 8
curler_pants_style  equ     curler_pants_color + 8

; Build the configuration screen and kickoff the first loop.
start_configuration subroutine
    jsr     stop_all_sounds
    sei                             ; Suspend interrupts.
    mva     #7,$ff                  ; Clear page 7, this is where all the
    jsr     clear_page              ; game configuration lives.
    mva     #$10,$ff                ; Clear the graphics areas.
    jsr     clear_page
    mva     #$20,$ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    mva     #$14,$ff                ; Clear the player areas.
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    mva     #5,title_blink_timer
    lda     <#main_config_list      ; Setup the display list.
    sta     sDLISTL
    sta     DLISTL
    lda     >#main_config_list
    sta     sDLISTH
    sta     DLISTH
    mwa     $1001,$fe
    mwa     title_text,$fc
    jsr     copy_text_full
    mwa     $2028,$fe
    mwa     select_length_text,$fc
    jsr     copy_text_full
    mwa     $2348,$fe
    mwa     build_id,$fc
    ldy     #39
.color_loop
    lda     $1000,y
    ora     #$80
    sta     $1000,y
    dey
    bpl     .color_loop
    jsr     copy_text_full
    mva     #0,sCOLBK
    sta     sCOLOR2
    sta     HPOSP0                  ; Move the players offscreen.
    sta     HPOSP1
    sta     HPOSP2
    sta     HPOSP3
    sta     title_rock_pos
    sta     HPOSM0                  ; Move the missiles offscreen.
    sta     HPOSM1
    sta     HPOSM2
    sta     HPOSM3
    sta     menu_type               ; Reset the menu type.
    sta     PRIOR                   ; Players in front.
    mva     #$0f,sCOLOR1
    mwa     configuration_vblank,VVBLKD
    mva     #$c0,NMIEN
    mva     #$ff,last_key_pressed
    mva     #240,title_rock_timer
    mva     #1,AUDCTL
    cli                             ; Enable interrupts.
    ; Fall through to the game length loop.

; Game length selection loop.
game_length_loop subroutine
    jsr     wait_for_vb
    lda     last_key_pressed
    bmi     game_length_loop
    beq     .ten_ends
    cmp     #$0e
    bne     .not_reset
    jmp     start_title
.not_reset
    cmp     #10
    bmi     .number_key
    mva     #$ff,last_key_pressed
    jmp     game_length_loop
.number_key
    sta     game_length
    clc
    adc     #$10
    sta     $2044
    jmp     .play_key_sound
.ten_ends
    lda     #10
    sta     game_length
    mva     #$11,$2044
    mva     #$10,$2045
.play_key_sound
    jsr     play_menu_sound
.clear_key
    mva     #$ff,last_key_pressed
    mwa     $2078,$fe
    mwa     game_type_text,$fc
    jsr     copy_text_full
    mva     #$a0,$fe
    mwa     one_one_one_text,$fc
    jsr     copy_text_full
    mva     #$c8,$fe
    mwa     one_one_two_text,$fc
    jsr     copy_text_full
    mva     #$f0,$fe
    mwa     two_two_two_text,$fc
    jsr     copy_text_full
    mwa     $2118,$fe
    mwa     two_two_four_text,$fc
    jsr     copy_text_full
    mva     #$40,$fe
    mwa     four_four_text,$fc
    jsr     copy_text_full
    mva     #$68,$fe
    mwa     one_cpu_text,$fc
    jsr     copy_text_full
    mva     #$90,$fe
    mwa     two_cpu_text,$fc
    jsr     copy_text_full
    mva     #$b8,$fe
    mwa     four_cpu_text,$fc
    jsr     copy_text_full
    mva     #$e0,$fe
    mwa     cpu_cpu_text,$fc
    jsr     copy_text_full
    ; Fall through to the next loop.

; Game type selection loop
game_type_loop subroutine
    jsr     wait_for_vb
    lda     last_key_pressed
    cmp     #$0e
    bne     .not_reset
    jmp     start_title
.not_reset
    cmp     #1
    bcc     game_type_loop
    cmp     #$a
    bcs     game_type_loop
    sta     game_type
    jsr     play_menu_sound
    mva     #$ff,last_key_pressed
    lda     #0                      ; Clear the player images.
    sta     $14a9
    sta     $14aa
    sta     $14ab
    sta     $15ac
    sta     $15ad
    sta     $16ae
    sta     $16af
    sta     $16b0
    sta     $17b1
    sta     $17b2
    sta     SIZEP0
    sta     SIZEP2
    sta     SIZEP1
    sta     SIZEP3
    sta     curler_hair_style
    sta     curler_hair_style + 1
    sta     curler_hair_style + 2
    sta     curler_hair_style + 3
    sta     curler_hair_style + 4
    sta     curler_hair_style + 5
    sta     curler_hair_style + 6
    sta     curler_hair_style + 7
    mva     #1,active_team
    lda     #rock_color_0
    sta     curler_jacket_color
    sta     curler_jacket_color + 1
    sta     curler_jacket_color + 2
    sta     curler_jacket_color + 3
    lda     #rock_color_1
    sta     curler_jacket_color + 4
    sta     curler_jacket_color + 5
    sta     curler_jacket_color + 6
    sta     curler_jacket_color + 7
    ldx     #$3d
    ldy     #7
.default_colors_loop
    lda     #$7a
    sta     curler_pants_color,y
    txa
    sta     curler_hair_color,y
    dey
    bpl     .default_colors_loop
    inc     menu_type
    jsr     ready_team_selection
    ; Fall through to the next loop.

; Configure/default menu loop
configure_default_loop subroutine
    jsr     wait_for_vb
    lda     last_key_pressed
    cmp     #$0e
    bne     .not_reset
    jmp     start_title
.not_reset
    cmp     #1
    beq     .default_team
    cmp     #2
    beq     .configure_team
    mva     #$ff,last_key_pressed
    jmp     configure_default_loop
.default_team
    jsr     play_menu_sound
    lda     active_team
    cmp     #1
    beq     .team_1_defaults
    mva     #1,curler_handedness + 6
    jmp     coin_toss_loop
.team_1_defaults
    mva     #1,curler_handedness + 2
    jmp     .next_team
.configure_team
    jsr     play_menu_sound
    lda     active_team
    cmp     #1
    beq     .configure_team_1
    mva     #4,active_player        ; Start with player 4 on team 2.
    jmp     .ready_configuration
.configure_team_1
    mva     #0,active_player        ; Start with player 0 on team 1.
.ready_configuration
    lda     #0                      ; Clear the old text.
    ldy     #39
.clear_text_loop
    sta     $2028,y
    sta     $2050,y
    sta     $2078,y
    dey
    bpl     .clear_text_loop
    jsr     wait_for_vb
    mwa     $2028,$fe               ; Ready the text.
    mwa     configure_shooter_text,$fc
    jsr     copy_text_full
    mva     #$84,$fe
    mwa     handedness_text,$fc
    jsr     copy_text_full
    mva     #$ac,$fe
    mwa     hairstyle_text,$fc
    jsr     copy_text_full
    mva     #$d4,$fe
    mwa     hair_color_text,$fc
    jsr     copy_text_full
    mva     #$fc,$fe
    mwa     jacket_color_text,$fc
    jsr     copy_text_full
    mwa     $2124,$fe
    mwa     pants_color_text,$fc
    jsr     copy_text_full
    mva     #$4c,$fe
    mwa     pants_style_text,$fc
    jsr     copy_text_full
    mva     #$74,$fe
    mwa     done_text,$fc
    jsr     copy_text_full
    jmp     ready_shooter_configuration
.next_team
    mva     #$ff,last_key_pressed
    inc     active_team
    inc     $2038                   ; Update the team number.
    jmp     configure_default_loop

; Common code for prepping the team selection.
ready_team_selection subroutine
    mva     #$20,$ff                ; Clear the text from pages 20 and 21.
    jsr     clear_page
    jsr     wait_for_vb
    inc     $ff
    jsr     clear_page
    jsr     wait_for_vb
    mwa     team_selection_text,$fc
    mwa     $2028,$fe
    jsr     copy_text_full
    mwa     default_team_text,$fc
    mva     #$50,$fe
    jsr     copy_text_full
    mva     #$78,$fe
    mwa     custom_team_text,$fc
    jsr     copy_text_full
    mva     #0,HPOSP1               ; Move the players/missiles off screen.
    sta     HPOSP2
    sta     HPOSP3
    sta     HPOSM1
    sta     HPOSM2
    sta     HPOSM3
    rts

; Set up a shooter for configuration.
ready_shooter_configuration subroutine
    jsr     wait_for_vb
    mwa     $2096,$fe               ; Set the default text options.
    mwa     right_text,$fc
    jsr     copy_text_full
    mva     #$be,$fe
    mwa     short_text,$fc
    jsr     copy_text_full
    inc     $ff
    mva     #$5e,$fe
    mwa     solid_text,$fc
    jsr     copy_text_full
    ldx     #$41                    ; Base address.
    ldy     active_player
    jsr     draw_curler_common
    mva     #$4b,HPOSP1
    mva     #$48,HPOSP2
    mva     #$46,HPOSP3
    mva     #$14,sCOLPM1
    mva     #$ff,sCOLPM3
    sta     SIZEM
    ldy     #5                      ; Create the color labels.
.color_loop
    lda     #12
    sta     $1351,y
    lda     #48
    sta     $1359,y
    lda     #192
    sta     $1361,y
    dey
    bpl     .color_loop
    mva     #$a9,HPOSM1
    sta     HPOSM2
    sta     HPOSM3
    ldy     active_player
    lda     curler_hair_color,y
    sta     sCOLPM1
    lda     curler_jacket_color,y
    sta     sCOLPM2
    lda     curler_pants_color,y
    sta     sCOLPM3
    mva     #$ff,last_key_pressed
    ; Fall through to the next loop.

; Shooter configuration loop.
configure_shooter_loop subroutine
    jsr     wait_for_vb
    lda     last_key_pressed
    beq     .next_shooter
    cmp     #$0e
    bne     .not_reset
    jmp     start_title
.not_reset
    cmp     #1
    bne     .not_handedness
    jmp     .change_handedness
.not_handedness
    cmp     #2
    bne     .not_hair_style
    jmp     .change_hair_style
.not_hair_style
    cmp     #3
    bne     .not_hair_color
    jmp     .change_hair_color
.not_hair_color
    cmp     #4
    bne     .not_jacket_color
    jmp     .change_jacket_color
.not_jacket_color
    cmp     #5
    bne     .not_pants_color
    jmp     .change_pants_color
.not_pants_color
    cmp     #6
    bne     .clear_key
    jmp     .change_pants_style
.play_key_sound
    jsr     play_menu_sound
.clear_key
    mva     #$ff,last_key_pressed
    jmp     configure_shooter_loop
.next_shooter
    jsr     play_menu_sound
    ldy     active_player
    cpy     #3
    bne     .check_increment
    jsr     ready_team_selection
    inc     active_team
    inc     $2038                   ; Update the team number.
    jmp     configure_default_loop
.check_increment
    cpy     #7
    bne     .increment_shooter
    jmp     coin_toss_loop
.increment_shooter
    inc     active_player
    inc     $203b                   ; Update the shooter number.
    mva     #$ff,last_key_pressed
    jmp     ready_shooter_configuration
.change_handedness
    ldy     active_player
    lda     curler_handedness,y
    eor     #1
    sta     curler_handedness,y
    beq     .to_right_handed
    mwa     left_text,$fc
    jmp     .update_hand_text
.to_right_handed
    mwa     right_text,$fc
.update_hand_text
    mwa     $2096,$fe
    jsr     copy_text_full
.redraw_curler
    ldx     #$41
    ldy     active_player
    jsr     draw_curler_common
    jmp     .play_key_sound
.change_hair_style
    ldy     active_player
    lda     curler_hair_style,y
    cmp     #4
    beq     .loop_hair_style
    clc
    adc     #1
    sta     curler_hair_style,y
    jmp     .update_hair_text
.loop_hair_style
    lda     #0
    sta     curler_hair_style,y
.update_hair_text
    bne     .not_short_hair
    mwx     short_text,$fc
    jmp     .apply_hair_text
.not_short_hair
    cmp     #1
    bne     .not_long_hair
    mwx     long_text,$fc
    jmp     .apply_hair_text
.not_long_hair
    cmp     #2
    bne     .not_tail_hair
    mwx     tail_text,$fc
    jmp     .apply_hair_text
.not_tail_hair
    cmp     #3
    bne     .not_friz_hair
    mwx     friz_text,$fc
    jmp     .apply_hair_text
.not_friz_hair
    mwx     afro_text,$fc
.apply_hair_text
    mwa     $20be,$fe
    jsr     copy_text_full
    jmp     .redraw_curler
.change_hair_color
    ldy     active_player
    lda     curler_hair_color,y
    clc
    adc     #16
    sta     curler_hair_color,y
    sta     sCOLPM1
    jmp     .play_key_sound
.change_jacket_color
    ldy     active_player
    lda     curler_jacket_color,y
    clc
    adc     #16
    sta     curler_jacket_color,y
    sta     sCOLPM2
    jmp     .play_key_sound
.change_pants_color
    ldy     active_player
    lda     curler_pants_color,y
    clc
    adc     #16
    sta     curler_pants_color,y
    sta     sCOLPM3
    jmp     .play_key_sound
.change_pants_style
    ldy     active_player
    lda     curler_pants_style,y
    eor     #1
    sta     curler_pants_style,y
    beq     .to_solid_pants
    mwa     norwegian_text,$fc
    jmp     .update_pants_text
.to_solid_pants
    mwa     solid_text,$fc
.update_pants_text
    mwa     $215e,$fe
    jsr     copy_text_full
    jmp     .redraw_curler

; Move the flipping coin up one pixel.
move_coin_up subroutine
    ldx     coin_low_byte           ; Get the current low byte.
.loop
    lda     $1400,x                 ; Move each pixel up.
    sta     $13ff,x
    lda     $1500,x
    sta     $14ff,x
    lda     $1600,x
    sta     $15ff,x
    lda     $1700,x
    sta     $16ff,x
    cpx     coin_high_byte          ; Stop when x has passed the last byte.
    beq     .continue
    bcs     .done
.continue
    inx
    jmp     .loop
.done
    dec     coin_low_byte           ; Adjust the bytes.
    dec     coin_high_byte
    rts

; Move the flipping coin down one pixel.
move_coin_down subroutine
    ldx     coin_high_byte          ; Get the current high byte.
.loop
    lda     $1400,x                 ; Move each pixel down.
    sta     $1401,x
    lda     $1500,x
    sta     $1501,x
    lda     $1600,x
    sta     $1601,x
    lda     $1700,x
    sta     $1701,x
    cpx     coin_low_byte           ; Stop when x has passed the first byte.
    bcc     .done
    dex
    jmp     .loop
.done
    inc     coin_low_byte           ; Adjust the bytes.
    inc     coin_high_byte
    rts

; Pre-game coin toss display loop.
coin_toss_loop subroutine
    mva     #0,HPOSM1               ; Get the missiles off screen.
    sta     HPOSM2
    sta     HPOSM3
    mva     #$20,$ff                ; Clear all the text.
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    mva     #$14,$ff                ; Clear the player areas.
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    mva     #$ff,sCOLPM0
    sta     sCOLPM1
    mva     #rock_color_1,sCOLPM2
    sta     sCOLPM3
    mva     #120,HPOSP0
    sta     HPOSP2
    mva     #128,HPOSP1
    sta     HPOSP3
    mva     #25,coin_countdown      ; Set the number of coin flips to do.
    mvx     #$80,coin_low_byte      ; Set the initial boundaries.
    mvy     #$91,coin_high_byte
    ldy     #0                      ; Graphics index.
.draw_coin_forward_loop
    lda     coin_frame_p0,y
    sta     $1400,x
    lda     coin_frame_p1,y
    sta     $1500,x
    lda     coin_frame_p2,y
    sta     $1600,x
    lda     coin_frame_p3,y
    sta     $1700,x
    iny
    inx
    cpy     #9                      ; Copy 9 bytes.
    bne     .draw_coin_forward_loop
.draw_coin_backward_loop
    dey                             ; Go the other way.
    bmi     .done_draw_coin
    lda     coin_frame_p0,y
    sta     $1400,x
    lda     coin_frame_p1,y
    sta     $1500,x
    lda     coin_frame_p2,y
    sta     $1600,x
    lda     coin_frame_p3,y
    sta     $1700,x
    inx
    jmp     .draw_coin_backward_loop
.done_draw_coin
    ldx     coin_low_byte
    ldy     coin_high_byte
    mva     #2,coin_timer           ; Two frames per vblank.
.coin_collapse_loop
    dec     coin_timer              ; Check the vblank timer.
    bne     .no_collapse_vblank
    jsr     wait_for_vb
    mva     #2,coin_timer
.no_collapse_vblank
    lda     $1400,x                 ; OR the edges with the succeeding bytes.
    ora     $1401,x
    sta     $1401,x
    pha                             ; Save this pattern for the reverse animation.
    lda     $1500,x
    ora     $1501,x
    sta     $1501,x
    pha
    lda     $1400,y
    ora     $13ff,y
    sta     $13ff,y
    lda     $1500,y
    ora     $14ff,y
    sta     $14ff,y
    lda     #0                      ; Blank the bytes that we moved from.
    sta     $1400,x
    sta     $1400,y
    sta     $1500,x
    sta     $1500,y
    sta     $1600,x
    sta     $1600,y
    sta     $1700,x
    sta     $1700,y
    inc     coin_low_byte
    dec     coin_high_byte
    inx
    lda     coin_high_byte
    sec
    sbc     coin_low_byte
    cmp     #1
    beq     .done_coin_collapse
    ldy     coin_high_byte
    jmp     .coin_collapse_loop
.done_coin_collapse
    mva     #2,coin_timer           ; Two frames per vblank.
    ldy     #8                      ; Reset the graphics index.
.coin_expand_loop
    dec     coin_timer
    bne     .no_expand_vblank
    jsr     wait_for_vb
    mva     #2,coin_timer
.no_expand_vblank
    lda     coin_frame_p0,y
    sta     $1400,x
    lda     coin_frame_p1,y
    sta     $1500,x
    lda     coin_frame_p2,y
    sta     $1600,x
    lda     coin_frame_p3,y
    sta     $1700,x
    ldx     coin_high_byte
    lda     coin_frame_p0,y
    sta     $1400,x
    lda     coin_frame_p1,y
    sta     $1500,x
    lda     coin_frame_p2,y
    sta     $1600,x
    lda     coin_frame_p3,y
    sta     $1700,x
    pla                             ; Get the composite pattern back.
    sta     $1501,x
    ldx     coin_low_byte
    sta     $14ff,x
    pla
    sta     $13ff,x
    ldx     coin_high_byte
    sta     $1401,x
    dey
    bmi     .coin_expand_done
    dec     coin_low_byte
    inc     coin_high_byte
    ldx     coin_low_byte
    jmp     .coin_expand_loop
.coin_expand_done
    ldx     #$ff                    ; Clear the stack, it's not quite
    txs                             ; empty at this point.
    ldx     coin_countdown          ; Check to see if the animation is done.
    dex
    beq     .coin_flip_over
    stx     coin_countdown          ; Move the coin appropriately.
    cpx     #8
    bmi     .down_fast
    cpx     #12
    bmi     .down_slow
    cpx     #15
    bmi     .update_coin_color      ; No movement.
    cpx     #18
    bmi     .up_slow
    jsr     move_coin_up
    jsr     move_coin_up
.up_slow
    jsr     move_coin_up
    jsr     move_coin_up
    jmp     .update_coin_color
.down_fast
    jsr     move_coin_down
    jsr     move_coin_down
.down_slow
    jsr     move_coin_down
    jsr     move_coin_down
.update_coin_color
    lda     sCOLPM2                 ; Flip the coin color.
    cmp     #rock_color_0
    beq     .to_rock_color_1
    lda     #rock_color_0
    jmp     .store_rock_color
.to_rock_color_1
    lda     #rock_color_1
.store_rock_color
    sta     sCOLPM2
    sta     sCOLPM3
    jmp     .done_draw_coin
.coin_flip_over
    mwa     hammer_select_text,$fc
    mwa     $2028,$fe
    jsr     copy_text_full
    mwa     throw_first_text,$fc
    mwa     $203c,$fe
    jsr     copy_text_full
    mwa     throw_last_text,$fc
    mwa     $2064,$fe
    jsr     copy_text_full
    lda     RANDOM
    and     #1
    sta     coin_toss_winner
    beq     .coin_winner_0
    inc     $202e                   ; Change the number to 2.
    lda     #rock_color_1           ; Set the coin color.
    jmp     .set_final_coin_color
.coin_winner_0
    lda     #rock_color_0           ; Set the coin color.
.set_final_coin_color
    sta     sCOLPM2
    sta     sCOLPM3
    ; Fall through to the input loop.

; Hammer selection input loop.
hammer_loop subroutine
    mva     #$ff,last_key_pressed
    lda     game_type
    cmp     #6                      ; Game mode less than 6, no CPU player.
    bmi     .human_select
    cmp     #9                      ; Game mode 9, no human players.
    beq     .cpu_select
    lda     coin_toss_winner
    beq     .human_select           ; Game mode 6-8, team 0 is human.
.cpu_select
    ldy     #60
    jsr     wait_for_y_frames       ; Delay a bit.
    lda     RANDOM
    and     #1                      ; Make a random choice.
    jmp     .set_hammer
.human_select
    jsr     wait_for_vb
    lda     last_key_pressed
    cmp     #$0e
    bne     .not_reset
    jmp     start_title
.not_reset
    cmp     #1
    beq     .shoot_first
    cmp     #2
    beq     .shoot_last
    jmp     hammer_loop
.shoot_first
    lda     coin_toss_winner
    eor     #1
    jmp     .set_hammer
.shoot_last
    lda     coin_toss_winner
.set_hammer
    sta     hammer_owner
    jsr     play_menu_sound
    jmp     start_game

; Common curler drawing code shared by the game.
; Expects the low byte of the base address in x,
; and the curler number in y.
draw_curler_common subroutine
    lda     curler_handedness,y     ; Check the handedness.
    sta     $ff                     ; Save it for later.
    bne     .left_handed            ; Copy in the opposite direction if lefty.
    lda     #0
    jmp     .prep_images
.left_handed
    lda     #6
.prep_images
    pha                             ; Save the read index.
    lda     curler_hair_style,y     ; Prepare the hair.
    beq     .hair_normal
    cmp     #1
    beq     .hair_long
    cmp     #2
    beq     .hair_tail
    cmp     #3
    beq     .hair_friz
    mwa     curler_hair_afro,$f0
    jmp     .hair_done
.hair_normal
    mwa     curler_hair_normal,$f0
    jmp     .hair_done
.hair_long
    mwa     curler_hair_long,$f0
    jmp     .hair_done
.hair_tail
    mwa     curler_hair_tail,$f0
    jmp     .hair_done
.hair_friz
    mwa     curler_hair_friz,$f0
.hair_done
    lda     curler_pants_style,y    ; Prepare the pants.
    beq     .pants_solid
    mwa     curler_p3_norwegian,$f2
    jmp     .start_draw
.pants_solid
    mwa     curler_p3,$f2
.start_draw
    pla                             ; Get the read index back.
    tay
.draw_loop
    lda     ($f0),y
    sta     $1500,x
    lda     curler_p2_rest,y
    sta     $1600,x
    lda     ($f2),y
    sta     $1700,x
    lda     $ff                     ; Adjust y based on handedness.
    bne     .left_handed_index
    cpy     #6
    beq     .done
    iny
    jmp     .continue_loop
.left_handed_index
    dey
    bmi     .done
.continue_loop
    inx
    jmp     .draw_loop
.done
    rts

; Build the rock that slides by at the offset in y.
build_menu_rock subroutine
    mva     #1,SIZEP0               ; Make the players double size.
    sta     SIZEP1
    sta     SIZEP2
    sta     SIZEP3
    lda     menu_rock               ; Start copying the image.
    sta     $1400,y
    lda     menu_rock + 1
    sta     $1401,y
    lda     menu_rock + 2
    sta     $1402,y
    lda     menu_rock + 3
    sta     $1503,y
    lda     menu_rock + 4
    sta     $1504,y
    lda     menu_rock + 5
    sta     $1605,y
    lda     menu_rock + 6
    sta     $1606,y
    lda     menu_rock + 7
    sta     $1607,y
    lda     menu_rock + 8
    sta     $1708,y
    lda     menu_rock + 9
    sta     $1709,y
    mva     #$36,sCOLPM0
    mva     #$06,sCOLPM1
    sta     sCOLPM3
    mva     #$0a,sCOLPM2
    rts

; Play the key acknowledgement tone.
play_menu_sound subroutine
    mwa     menu_selection_sound,$fe
    jsr     play_sound_1
    rts

; Deferred vblank handler.
configuration_vblank subroutine
    jsr     process_sounds
    mva     #0,ATRACT               ; Kill the attract timer.
    ldy     key_debounce            ; Process the debounce timer.
    beq     .no_debounce
    dec     key_debounce
.no_debounce
    dec     title_blink_timer
    bne     .no_title_blink
    inc     sCOLOR3
    inc     sCOLOR3
    mva     #5,title_blink_timer
.no_title_blink
    lda     menu_type               ; Only do the cute rock thing
    bne     .no_rock_slide          ; on menu type 0.
    lda     title_rock_timer
    beq     .handle_rock_slide
    dec     title_rock_timer
    jmp     .no_rock_slide
.handle_rock_slide
    ldy     title_rock_pos
    bne     .already_in_motion
    ldy     #$a9
    jsr     build_menu_rock
    ldy     #0
.already_in_motion
    iny
    sty     HPOSP0
    sty     HPOSP1
    sty     HPOSP2
    sty     HPOSP3
    sty     title_rock_pos
    bne     .no_rock_slide
    mva     #240,title_rock_timer
.no_rock_slide
    jmp     ENDINT

