; This file is part of RealSports Curling.

; RealSports Curling is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; RealSports Curling is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with RealSports Curling.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2018 Average Software.

; Routines for the CPU player

; CPU state constants.
cpu_collect_data            equ $0
cpu_determine_shot_class    equ $1
cpu_timer_length            equ 60
cpu_takeout_timer_length    equ 40
cpu_sweep_mode_none         equ $ff
cpu_sweep_mode_light        equ $0
cpu_sweep_mode_medium       equ $1
cpu_sweep_mode_heavy        equ $2
cpu_sweep_light_base        equ 40
cpu_sweep_medium_base       equ 50
cpu_sweep_heavy_base        equ 60
cpu_sweep_light_timer       equ 3
cpu_sweep_medium_timer      equ 2
cpu_sweep_heavy_timer       equ 1

; CPU data values.
cpu_state                   equ $6a0
cpu_my_rocks_in_house       equ $6a1
cpu_their_rocks_in_house    equ $6a2
cpu_closest_rock            equ $6a3
cpu_closest_rock_2          equ $6a4
cpu_color_id                equ $6a5
cpu_weight_mode             equ $6a6
cpu_target_x                equ $6a7
cpu_target_y                equ $6a8
cpu_aim                     equ $6a9
cpu_stick_base              equ $6aa
cpu_shot_base               equ $6ab
cpu_input_timer             equ $6ac
cpu_closest_distance        equ $6ad
cpu_weight                  equ $6ae
cpu_curl                    equ $6af
cpu_sweep_mode              equ $6b0

; Reset the CPU state prior to a shot.
reset_cpu_player subroutine
    mva     #cpu_collect_data,cpu_state
    sta     cpu_my_rocks_in_house
    sta     cpu_their_rocks_in_house
    mva     #$ff,cpu_closest_rock
    sta     cpu_closest_rock_2
    mva     #cpu_timer_length,cpu_input_timer
    jsr     get_handedness
    beq     .right_handed
    lda     #109
    ldy     #39
    jmp     .store_handedness_values
.right_handed
    lda     #96
    ldy     #26
.store_handedness_values
    sta     cpu_stick_base
    sty     cpu_shot_base
    rts

; Routine for getting the CPU's aim value.
get_cpu_aim subroutine
    lda     cpu_state               ; First, see if data needs to be
    bne     .data_ready             ; collected.
    jmp     .collect_cpu_data
.data_ready
    lda     cpu_aim
    jmp     aim_reentry
.collect_cpu_data
    mva     #$ff,cpu_closest_distance
    sta     cpu_closest_distance + 1
    lda     current_rock_number     ; Store a bit that represents which
    and     #1                      ; color the CPU player is.
    sta     cpu_color_id
    ldy     #15                     ; Loop through the rocks, collecting
.rock_count_loop                    ; counts of the rocks in the house.
    lda     rock_progress_high,y
    beq     .next_rock              ; High byte must be 1.
    lda     rock_progress,y         ; Check to see if this rock is in
    jsr     rock_in_house           ; the house.
    bne     .next_rock              ; It isn't.
    tya                             ; See who this rock belongs to.
    and     #1                      ; Mask to the end bit.
    cmp     cpu_color_id            ; Check against the active player bit.
    beq     .my_rock_in_house
    inc     cpu_their_rocks_in_house
    jmp     .check_distance
.my_rock_in_house
    inc     cpu_my_rocks_in_house
.check_distance
    jsr     distance_from_button    ; Find out how close this rock is.
    mva     $fe,$fc                 ; Set up the comparison.
    mva     $ff,$fd
    mva     cpu_closest_distance,$fa
    mva     cpu_closest_distance + 1,$fb
    jsr     compare_16
    cmp     #1
    bne     .next_rock
    lda     cpu_closest_rock        ; The previous rock is now
    sta     cpu_closest_rock_2      ; 2nd closest.
    sty     cpu_closest_rock        ; This rock is closer.
    mva     $fc,cpu_closest_distance
    mva     $fd,cpu_closest_distance + 1
.next_rock
    dey
    bpl     .rock_count_loop
    lda     RANDOM                  ; Pick a random weight.
    and     #$1f                    ; Mask down to 31.
    clc
    adc     #weight_lower_bound
    sta     cpu_weight
    lda     cpu_closest_rock        ; See who has the scoring rock.
    bmi     .no_scoring_rock
    and     #1
    cmp     cpu_color_id
    beq     .cpu_has_closest
    lda     cpu_closest_rock_2      ; See who has the second rock.
    bmi     .no_second_closest
    and     #1
    cmp     cpu_color_id
    beq     .no_second_closest
    jmp     .cpu_draw
.no_second_closest
    jmp     .cpu_takeout
.cpu_has_closest
    lda     cpu_closest_rock_2      ; See who has the second rock.
    bpl     .closest_rock_2_exists
    jmp     .cpu_draw
.closest_rock_2_exists
    and     #1
    cmp     cpu_color_id
    beq     .cpu_guard
    jmp     .cpu_draw
.no_scoring_rock
    lda     current_rock_number     ; Draw on the last shot.
    cmp     #14
    bpl     .cpu_draw
    lda     RANDOM                  ; 50/50 draw/guard.
    and     #1
    beq     .cpu_draw
.cpu_guard
    mva     #weight_mode_guard,cpu_weight_mode
    lda     cpu_weight
    cmp     #100                    ; Pick a sweep mode.
    bmi     .heavy_guard_sweep
    lda     #cpu_sweep_mode_light
    jmp     .set_guard_sweep
.heavy_guard_sweep
    lda     #cpu_sweep_mode_heavy
.set_guard_sweep
    sta     cpu_sweep_mode
    lda     cpu_closest_rock        ; See who has the closest rock.
    bmi     .free_guard             ; Nobody, just place a guard.
    and     #1
    cmp     cpu_color_id
    bne     .free_guard             ; Other player, place a free guard.
    ldy     cpu_closest_rock        ; Get the rock's vertical.
    lda     rock_vertical,y
    sta     cpu_target_y            ; Try to line up with it.
    lda     #10                     ; Aim for the front of the sheet.
    sta     cpu_target_x
    jmp     .finalize_aim
.free_guard
    lda     RANDOM                  ; Choose between a center and
    and     #1                      ; corner guard.
    beq     .center_guard
    lda     RANDOM                  ; Choose a side.
    and     #1
    beq     .top_guard
    lda     #55                     ; Aim low.
    sta     cpu_target_y
    lda     #10                     ; Aim shallow.
    sta     cpu_target_x
    jmp     .finalize_aim
.center_guard
    lda     #35                     ; Aim for the middle.
    sta     cpu_target_y
    lda     #10                     ; Aim shallow.
    sta     cpu_target_x
    jmp     .finalize_aim
.top_guard
    lda     #10                     ; Aim high.
    sta     cpu_target_y
    lda     #10                     ; Aim shallow.
    sta     cpu_target_x
    jmp     .finalize_aim
.cpu_draw
    mva     #weight_mode_draw,cpu_weight_mode
    mva     #cpu_sweep_mode_light,cpu_sweep_mode
    lda     cpu_closest_rock        ; Who owns the closest rock?
    bmi     .cpu_draw_button        ; Nobody?  Draw for a point.
    and     #1
    cmp     cpu_color_id
    beq     .cpu_draw_split         ; Split the house.
.cpu_draw_button
    lda     #button_left            ; Shoot for the button.
    sta     cpu_target_x
    lda     #35                     ; Aim center.
    sta     cpu_target_y
    jmp     .finalize_aim
.cpu_draw_split
    lda     #35
    sta     cpu_target_x            ; Aim center.
    lda     RANDOM                  ; Pick a side.
    and     #1
    beq     .cpu_split_up
    lda     #55                     ; Aim low.
    sta     cpu_target_y
    jmp     .finalize_aim
.cpu_split_up
    lda     #10                     ; Aim high.
    sta     cpu_target_y
    jmp     .finalize_aim
.cpu_takeout
    mva     #weight_mode_takeout,cpu_weight_mode
    mva     #cpu_sweep_mode_heavy,cpu_sweep_mode
    ldy     cpu_closest_rock        ; Aim for the closest rock.
    lda     rock_progress,y
    sta     cpu_target_x
    lda     rock_vertical,y
    sta     cpu_target_y
.finalize_aim
    lda     cpu_target_y            ; Jitter the vertical aim.
    cmp     #3                      ; If less than 3, don't jitter
    bmi     .no_y_jitter            ; to avoid underflow.
    tax
    jsr     get_aim_jitter
    clc
    adc     cpu_target_y
.no_y_jitter
    sec                             ; Get the relative vertical distance
    sbc     cpu_shot_base           ; from the shot position.
    bpl     .positive_aim
    tax
    lda     #-1                     ; Store a sign flag.
    pha
    mva     #130,cpu_curl           ; Curl down.
    txa
    jsr     absolute_value          ; Convert to positive for ease of math.
    jmp     .finished_aim_sign
.positive_aim
    tax
    lda     #1                      ; Store a sign flag.
    pha
    mva     #70,cpu_curl            ; Curl up.
    txa
.finished_aim_sign
    sta     $ff                     ; Multiply by the distance to the
    lda     #hogline_distance       ; front hogline.
    jsr     multiply
    sta     $fa                     ; Store the result.
    stx     $fb
    mva     #1,$fd                  ; Set up the divisor.
    mva     cpu_target_x,$fc
    jsr     divide_16
    pla                             ; Check the sign of the result.
    bmi     .aim_up
    lda     cpu_stick_base          ; Add the result to the base aim.
    clc
    adc     $fe
    adc     #3                      ; Add a little more to allow for curl.
    jmp     .store_cpu_aim
.aim_up
    lda     cpu_stick_base          ; Subtract the result from the base aim.
    sec
    sbc     $fe
    sbc     #3                      ; Subtract a little more to allow for curl.
.store_cpu_aim
    sta     cpu_aim
    inc     cpu_state
    jmp     aim_reentry

; Routine for getting the CPU's trigger value.
get_cpu_trig subroutine
    dec     cpu_input_timer
    bne     .not_yet
    mva     #cpu_timer_length,cpu_input_timer
    mva     #1,trigger_buffer
    lda     shot_mode
    cmp     #shot_mode_weight
    bne     .not_weight
    lda     cpu_weight_mode
    cmp     #weight_mode_takeout
    bne     .no_sweep_prep
    mva     #cpu_takeout_timer_length,cpu_input_timer
    jmp     .no_sweep_prep
.not_weight
    cmp     #shot_mode_curl
    bne     .no_sweep_prep
    jsr     cpu_prepare_sweep
.no_sweep_prep
    lda     #0
.not_yet
    jmp     trig_reentry

; Routine for getting the CPU's weight value.
get_cpu_weight subroutine
    lda     cpu_weight
    jmp     weight_reentry

; Routine for getting the CPU's vertical curl value.
get_cpu_curl subroutine
    lda     cpu_curl
    jmp     curl_reentry

; Routine for reading the CPU's keypad.
get_cpu_keypress subroutine
    lda     cpu_weight_mode
    jmp     key_reentry

; Routine for getting the CPU's sweeping input.
cpu_sweep subroutine
    dec     cpu_input_timer
    bne     .not_yet
    lda     RANDOM                  ; Prepare a random jitter.
    and     #$0f                    ; Mask off the top four bits.
    clc
    ldy     cpu_sweep_mode
    bmi     .no_sweep
    beq     .light_sweep
    cpy     #cpu_sweep_mode_medium
    beq     .medium_sweep
    adc     #cpu_sweep_heavy_base
    ldy     #cpu_sweep_heavy_timer
    jmp     .reset_timer
.no_sweep
    lda     #0
    ldy     #60                     ; Arbitrary number.
    jmp     .reset_timer
.light_sweep
    adc     #cpu_sweep_light_base
    ldy     #cpu_sweep_light_timer
    jmp     .reset_timer
.medium_sweep
    adc     #cpu_sweep_medium_base
    ldy     #cpu_sweep_medium_timer
.reset_timer
    sty     cpu_input_timer
.not_yet
    jmp     sweep_reentry

; Prepare the sweep values.
cpu_prepare_sweep subroutine
    lda     cpu_sweep_mode
    beq     .sweep_light
    cmp     #cpu_sweep_mode_medium
    beq     .sweep_medium
    lda     #cpu_sweep_heavy_timer
    jmp     .store_timer
.sweep_light
    lda     #cpu_sweep_light_timer
    jmp     .store_timer
.sweep_medium
    lda     #cpu_sweep_medium_timer
.store_timer
    sta     cpu_input_timer
    rts

; Generate a random aim jitter of -2 .. 2.
get_aim_jitter subroutine
    lda     RANDOM                  ; Start with a random value.
    bmi     .negative_jitter        ; Get a direction.
    ldy     #1                      ; Positive jitter.
    jmp     .get_magnitude
.negative_jitter
    ldy     #-1                     ; Negative jitter.
.get_magnitude
    and     #$03                    ; Mask off all but the last 2 bits.
    beq     .no_jitter              ; On 0, the aim is true.
    cmp     #3                      ; On 3, the aim is also true.
    beq     .no_jitter
    cpy     #1                      ; If jitter is positive, we're done.
    beq     .no_jitter
    eor     #$ff                    ; Convert to negative.
    clc
    adc     #1
.no_jitter
    rts

