; This file is part of RealSports Curling.

; RealSports Curling is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; RealSports Curling is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with RealSports Curling.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2018 Average Software.

; Debug related data and utilities

; Debugging routine, just enters an infinite loop.
debug_break subroutine
    jmp     debug_break

; Debugging data.
rock_progress_debug_data
    .byte   251                     ; Rock progress low bytes.
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00

rock_progress_high_debug_data
    .byte   $00                     ; Rock progress high bytes.
    .byte   $00
    .byte   $00 
    .byte   $00 
    .byte   $00 
    .byte   $00 
    .byte   $00 
    .byte   $00 
    .byte   $00 
    .byte   $00 
    .byte   $00 
    .byte   $00 
    .byte   $00 
    .byte   $00 
    .byte   $00 
    .byte   $00

rock_vertical_debug_data
    .byte   $1a                     ; Rock vertical bytes.
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00
    .byte   $00

rock_number_debug_data
    .byte   $0f                     ; Starting rock number.

starting_hammer
    .byte   $01                     ; Which team has the hammer?

; This subroutine puts the debug data in place
; and begins the game from that point.
apply_debug_data subroutine
    lda     #rock_data_1
    sta     $f0
    lda     #rock_team
    sta     $f2
    ldy     #0
    sty     $f1
.rock_loop
    ldy     $f1
    lda     rock_progress_high_debug_data,y
    beq     .no_high_byte
    sta     rock_progress_high,y
    ldy     $f0
    lda     $f2
    eor     #rock_team
    sta     $f2
    sta     rock_flags,y
    lda     #$60
    sta     rock_address,y
    lda     #$20
    sta     rock_address + 1,y
    lda     #0
    sta     rock_offset,y
    jmp     .handle_low_byte
.no_high_byte
    ldy     $f0
    lda     <#coordinate_base
    sta     rock_address,y
    lda     >#coordinate_base
    sta     rock_address + 1,y
.handle_low_byte
    ldy     $f1
    lda     rock_progress_debug_data,y
    sta     rock_progress,y
    tax
    jsr     apply_rock_progress
    ldy     $f1
    lda     rock_vertical_debug_data,y
    sta     rock_vertical,y
    tax
    jsr     apply_rock_vertical
    ldy     $f1
    lda     rock_progress_high,y
    bne     .draw_rock
    lda     rock_progress,y
    bne     .draw_rock
    lda     rock_vertical,y
    beq     .no_draw_rock
.draw_rock
    ldy     $f0
    lda     rock_address,y
    sta     $fe
    lda     rock_address + 1,y
    sta     $ff
    lda     $f1
    and     #1
    tax
    lda     rock_offset,y
    tay
    lda     #0
    jsr     draw_rock
    ldy     $f1
.no_draw_rock
    iny
    cpy     #15
    beq     .rock_loop_done
    sty     $f1
    lda     $f0
    clc
    adc     #rock_data_size
    sta     $f0
    jmp     .rock_loop
.rock_loop_done
    mvy     rock_number_debug_data,current_rock_number
    beq     .pointer_ok
    lda     #rock_data_1
    clc
.pointer_loop
    dey
    bne     .continue
    sta     current_rock
    jmp     .pointer_ok
.continue
    adc     #rock_data_size
    jmp     .pointer_loop
.pointer_ok
    mva     starting_hammer,hammer_owner
    rts

; Apply horizontal progress to debug rocks.
apply_rock_progress subroutine
    cpx     #0
    bne     .do_progress
    rts
.do_progress
    ldy     $f0
.loop
    cpx     #4
    bcc     .apply_offset
    jsr     .apply_byte
    dex
    dex
    dex
    dex
    jmp     .loop
.apply_offset
    stx     $ff
    lda     rock_offset,y
    clc
    adc     $ff
    cmp     #4
    bmi     .done
    sec
    sbc     #4
    sta     rock_offset,y
    jsr     .apply_byte
    rts
.done
    sta     rock_offset,y
    rts

.apply_byte
    lda     rock_address,y
    clc
    adc     #1
    sta     rock_address,y
    lda     rock_address + 1,y
    adc     #0
    sta     rock_address + 1,y
    rts

apply_rock_vertical subroutine
    cpx     #0
    bne     .do_progress
    rts
.do_progress
    ldy     $f0
.loop
    dex
    cpx     #$ff
    beq     .done
    lda     rock_address,y
    clc
    adc     #bytes_per_row
    sta     rock_address,y
    lda     rock_address + 1,y
    adc     #0
    sta     rock_address + 1,y
    jmp     .loop
.done
    rts

