; This file is part of RealSports Curling.

; RealSports Curling is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; RealSports Curling is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with RealSports Curling.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2018 Average Software.

; Main game routines

; Ice color value.
ice_color           equ     $d

; Rock color values.
rock_color_0        equ     $db
rock_color_1        equ     $3a

; Scoreboard base addresses.
score_base_label    equ     $1029
score_base_0        equ     $1078
score_base_1        equ     $10c8

; Addresses for the rock to be shot.
throw_address_left  equ     $2cc5
throw_address_right equ     $288e

; Base address for the ice coordinates.
coordinate_base     equ     $2020

; Button coordinates.
button_left         equ     $2c
button_right        equ     $2d
button_bottom       equ     $21
button_top          equ     $20

; Base addresses for the aim assist scales.
aim_scale_near      equ     $2039
aim_scale_far       equ     $205c

; End addresses for the weight meter.
weight_meter_left   equ     $235c
weight_meter_right  equ     weight_meter_left + 5

; Start address for the weight description text.
weight_text         equ     weight_meter_left - bytes_per_row * 6

; Number of frames to wait before polling the sweep values.
sweep_delay         equ     #4

; Weight throw ranges.
guard_low           equ     68
guard_high          equ     76
guard_range         equ     #guard_high - #guard_low
guard_ratio         equ     [#weight_upper_bound - #weight_lower_bound] / #guard_range
draw_low            equ     75
draw_high           equ     80
draw_range          equ     #draw_high - #draw_low
draw_ratio          equ     [#weight_upper_bound - #weight_lower_bound] / #draw_range
takeout_low         equ     80
takeout_high        equ     100
takeout_range       equ     #takeout_high - #takeout_low
takeout_ratio       equ     [#weight_upper_bound - #weight_lower_bound] / #takeout_range

; Base address for the curl arrow.
curl_arrow_base     equ     $294a

; Base addresses for the status rocks.
p0_status_base      equ     $3750
p1_status_base      equ     $3768

; Color values for the rings.
outer_ring_color    equ     $80
inner_ring_color    equ     $30

; Base player location for the rings.
ring_base           equ     175

; Hog line missile location.
front_hog_line      equ     ring_base - 37

; Back line missile location.
back_line           equ     ring_base + 30

; End-cap line for the scoreboard location.
end_cap_line        equ     208

; Bytes per sheet row.
bytes_per_row       equ     83

; Distance to the front hogline.
hogline_distance    equ     70

; Distance to the aim cursor.
aim_distance        equ     99

; Minimum distance a rock must travel to cross
; the far hogline.
minimum_shot        equ     251

; Force required to move a rock.
rest_friction       equ     20
v_rest_friction     equ     20

; Delay time between shots.
shot_end_delay_time equ     120

; Rock data fields.
rock_address        equ     0
rock_offset         equ     2
rock_handle         equ     3
rock_h_speed        equ     4
rock_v_speed        equ     5
rock_r_speed        equ     6
rock_h_subpixel     equ     7
rock_v_subpixel     equ     8
rock_r_time         equ     9
rock_flags          equ     10
rock_data_size      equ     11

; Bit fields for the rock flags.
rock_team           equ     1
rock_rotation       equ     2
rock_v_direction    equ     4

; Shot mode constants.
shot_mode_none      equ     0
shot_mode_aim       equ     1
shot_mode_weight    equ     2
shot_mode_curl      equ     3

; Weight mode constants.
weight_mode_guard   equ     1
weight_mode_draw    equ     2
weight_mode_takeout equ     3

; Joystick bounds constants.
aim_lower_bound     equ     78
aim_upper_bound     equ     123
far_lower_bound_r   equ     87
far_upper_bound_r   equ     108
far_lower_bound_l   equ     96
far_upper_bound_l   equ     117
weight_lower_bound  equ     83
weight_upper_bound  equ     115

; Trigger debounce length.
trigger_debounce    equ     10

; Status blink timer length.
blink_length        equ     30

; Rock data storage.
rock_data_1         equ     $19
; List of rocks that need processing.
rock_list           equ     $d0
; Overflow address for the rock list.
rock_overflow       equ     rock_data_1 + rock_data_size * 16
; Aim cursor position, two byte address.
; $2031 is the upper boundary, $3544 the lower.
; $2a91 is the center.
aim_cursor          equ     $e0
; Current shot mode.
shot_mode           equ     $e2
; Last joystick position.
last_stick          equ     $e3
; Trigger buffer for preventing repeats.
trigger_buffer      equ     $e4
; De-bounce timer for the trigger.
trigger_timer       equ     $e5
; The address of the rock currently being thrown.
current_rock        equ     $e6
; The blink timer for the rock display at the bottom.
blink_timer         equ     $e7
; The current scroll value.
current_scroll      equ     $e8
; The pointer offset into the scroll area of the next display list.
scroll_list_offset  equ     $e9
; Lower sheet colors used in interrupts.
pm1_low_color_2     equ     $ea
pm2_low_color_2     equ     $eb
pm1_low_color_3     equ     $ec
pm2_low_color_3     equ     $ed
; Second position for two-stick sweeping.
last_stick_2        equ     $ee

; Player/missile positions for the bottom sheet.
p0_low_h            equ     $300
p1_low_h            equ     $301
p2_low_h            equ     $302
p3_low_h            equ     $303
m0_low_h            equ     $304
m1_low_h            equ     $305
m2_low_h            equ     $306
m3_low_h            equ     $307
; Player missile colors for the bottom sheet.
; PM0 is always blue, so no need for that.
; PM3 only has one color.
pm1_low_color_1     equ     $308
pm2_low_color_1     equ     $309
pm3_low_color       equ     $30a
; Counters for ice friction on the rocks.
rock_friction       equ     $310
; 16-bit counter for curl application on the shooter.
rock_curl           equ     $320
; Horizontal tracking for each rock, 16-bit.
rock_progress       equ     $322
rock_progress_high  equ     rock_progress + 16
; Vertical tracking for each rock.
rock_vertical       equ     rock_progress_high + 16
; Delay counter for the between shot pause.
shot_end_delay      equ     rock_vertical + 16
; Flag that allows rocks that strike a rock beyond the
; hogline to remain in play.
valid_rock_struck   equ     shot_end_delay + 1
; The number of the rock currently being thrown.
current_rock_number equ     valid_rock_struck + 1
; The current end number.
current_end         equ     current_rock_number + 1
; The horizontal location of the scoreboard hammer.
hammer_hpos         equ     current_end + 1
; The team that has the hammer.
hammer_owner        equ     hammer_hpos + 1
; The current pebble friction value.
pebble_friction     equ     hammer_owner + 1
; The frame count for pebble increases.
pebble_frames       equ     pebble_friction + 1
; The last keypad key pressed by the user.
last_key_pressed    equ     pebble_frames + 1
; The current weight mode.
weight_mode         equ     last_key_pressed + 1
; Address for the far aim cursor.
aim_cursor_far      equ     weight_mode + 1
; Sweep read counter.
sweep_timer         equ     aim_cursor_far + 2
; Sweep value storage.
sweep_value         equ     sweep_timer + 1
; Input vectors.
aim_value_vector    equ     sweep_value + 1
aim_trig_vector     equ     aim_value_vector + 2
weight_value_vector equ     aim_trig_vector + 2
curl_value_vector   equ     weight_value_vector + 2
key_value_vector    equ     curl_value_vector + 2
sweep_value_vector  equ     key_value_vector + 2
; Pot and trigger offsets.
shooter_vpot        equ     sweep_value_vector + 2
shooter_hpot        equ     shooter_vpot + 1
shooter_trig        equ     shooter_hpot + 1
sweeper_vpot_1      equ     shooter_trig + 1
sweeper_vpot_2      equ     sweeper_vpot_1 + 1
; PRIOR and SIZEP2 values for the lower sheet.
prior_low           equ     sweeper_vpot_2 + 1
sizep2_low          equ     prior_low + 1
; Reference address for drawing the "arm extended" image.
curler_p2_address   equ     sizep2_low + 1

; Scratch area for calculating the rock distances for scoring.
rock_scoring        equ     $600
; Storage for rocks that may need to be replaced due
; to a free guard zone violation.  Stores a complete
; rock record, plus the 16-bit horizontal progress
; and 8-bit vertical location.
fgz_rock_0          equ     $640
fgz_rock_1          equ     fgz_rock_0 + rock_data_size + 3
fgz_rock_2          equ     fgz_rock_1 + rock_data_size + 3
fgz_rock_3          equ     fgz_rock_2 + rock_data_size + 3

; Collision utility list.
active_rock_list    equ     $3f00

; Set the initial game parameters and kick off the game loop.
; The ice sheet lines start at $201d so that the 4k boundary ($3000)
; will align perfectly with the left edge of the sheet.
start_game subroutine
    mwa     ENDINT,VVBLKD           ; Shut down the vblank interrupt.
    mva     #$14,$ff                ; Clear the player graphics.
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    mva     #0,HPOSP0               ; Get the players off screen.
    sta     HPOSP1
    sta     HPOSP2
    sta     HPOSP3
    sta     SIZEM                   ; Go to normal size missiles.
    mva     #$70,$400               ; Start building the display lists.
    sta     $401                    ; 24 blank scanlines for the overscan.
    sta     $402
    mva     #$c2,$403               ; A line of mode 2, with an interrupt.
    mwa     $1000,$404              ; Start screen RAM at $1000.
    mwa     [$201d + bytes_per_row - 40],$fe
    ldy     #0                      ; 69 lines of mode e.
.top_area_loop
    lda     #$4e                    ; Load memory on every line.
    sta     $406,y
    iny
    lda     $fe                     ; Write the memory address.
    sta     $406,y
    iny
    lda     $ff
    sta     $406,y
    clc                             ; Update the address.
    lda     $fe
    adc     #bytes_per_row
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    iny
    cpy     #69 * 3
    bne     .top_area_loop          ; Quit after 69 lines.
    mva     #$ce,$4d5               ; One more e, with an interrupt.
    sta     $406 + 24 * 3           ; Set two more interrupts for
    sta     $406 + 44 * 3           ; the ring colors.
    sta     $406 + 33 * 3           ; Set two interrupts for the
    sta     $406 + 35 * 3           ; center line.
    mva     $fe,$4d6                ; Set the address for the last
    mva     $ff,$4d7                ; ice sheet line.
    mva     #$42,$4d8               ; Top line of the score sheet.
    mwa     $1028,$4d9
    mva     #$02,$4dc               ; Two more text lines with e in between.
    mva     #$82,$4de               ; Interrupt here to position the lower P/Ms.
    mva     #$0e,$4db
    sta     $4dd
    mva     #$8e,$4df               ; Interrupt on the last score line.
    mwa     $201d,$fe               ; Start the bottom ice sheet on the far left.
    ldy     #0                      ; 69 more rows.
.bottom_area_loop
    lda     #$5e                    ; Load memory and hscroll on every line.
    sta     $4e0,y
    iny
    lda     $fe                     ; Write the memory address.
    sta     $4e0,y
    iny
    lda     $ff
    sta     $4e0,y
    clc                             ; Update the address.
    lda     $fe
    adc     #bytes_per_row
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    iny
    cpy     #69 * 3
    bne     .bottom_area_loop       ; Quit after 69 lines.
    mva     #$de,$5af               ; One more e, with an interrupt and scroll.
    sta     $4e0 + 23 * 3           ; Set four more interrupts for
    sta     $4e0 + 45 * 3           ; the ring colors.
    sta     $4e0 + 24 * 3           ; These need more interrupts because the
    sta     $4e0 + 46 * 3           ; scroll processing eats some cycles.
    sta     $4e0 + 58 * 3           ; Set interrupts to position the sweep gauge.
    sta     $4e0 + 60 * 3
    sta     $4e0 + 33 * 3           ; Set two interrupts for the
    sta     $4e0 + 35 * 3           ; center line.
    mva     $fe,$5b0                ; Set the address for the last
    mva     $ff,$5b1                ; ice sheet line.
    mva     #$4e,$5b2               ; Set the beginning of the bottom area.
    mwa     $3700,$5b3              ; Start at $3700.
    ldy     #15                     ; 16 more rows of e.
    lda     #$0e
.status_area_loop
    sta     $5b5,y
    dey
    bpl     .status_area_loop
    mva     #$8e,$5b8               ; Set interrupts for status coloring.
    sta     $5ba
    mva     #$41,$5c5               ; Jump instruction.
    mwa     $400,$5c6               ; Go back to $400.
    ldy     #0                      ; Make another copy of the display list.
.copy_dlist_loop
    lda     $400,y                  ; This is used for scrolling performance.
    sta     $800,y
    lda     $500,y
    sta     $900,y
    iny
    bne     .copy_dlist_loop
    mwa     $800,$9c6               ; Change the copy's reset address.
    clc                             ; Build the scoreboard.
    lda     #$11                    ; Start with '1'.
    ldy     #0
    ldx     game_length             ; Get the number of ends.
.end_number_loop
    sta     score_base_label,y
    adc     #1
    iny
    iny
    iny
    iny
    dex
    bne     .end_number_loop
    ldx     game_length
    cpx     #10
    bne     .no_tenth_end
    dey
    dey
    dey                             ; Bring the y index back.
    lda     #$10                    ; Put the 10 on the last column.
    sta     score_base_label,y
    dey
    lda     #$11
    sta     score_base_label,y
.no_tenth_end
    lda     #124                    ; Put the | separators in.
    ldy     #0
    ldx     game_length
.separator_loop
    sta     score_base_0,y
    sta     score_base_1,y
    iny
    iny
    iny
    iny
    dex
    bpl     .separator_loop
    lda     #$aa                    ; Now draw the horizontal lines.
    ldy     #0
    ldx     game_length
.line_loop
    sta     $1050,y
    sta     $1051,y
    sta     $1052,y
    sta     $1053,y
    sta     $10a0,y
    sta     $10a1,y
    sta     $10a2,y
    sta     $10a3,y
    sta     $10f0,y
    sta     $10f1,y
    sta     $10f2,y
    sta     $10f3,y
    iny
    iny
    iny
    iny
    dex
    bne     .line_loop
    ldx     game_length
    cpx     #10
    beq     .no_line_extension
    lda     #$a0
    sta     $1050,y                 ; Need a little bit more.
    sta     $10a0,y
    sta     $10f0,y
.no_line_extension
    lda     #$0a                    ; These are the two pixels on the left.
    sta     $1050
    sta     $10a0
    sta     $10f0
    mwa     score_flag,$fe          ; Set up the team flags on the scoreboard.
    mwa     $1677,$fc
    mva     #8,$fb
    jsr     copy_memory
    mwa     $1780,$fc
    jsr     copy_memory
    mwa     $a000,VDSLST            ; Set the initial interrupt.
    mwa     game_vblank,VVBLKD      ; Set the vblank interrupt.
    mva     #0,sCOLBK               ; Start the background color at black.
    sta     sCOLOR2                 ; Black text background.
    sta     sDLISTL                 ; Low byte of the display list.
    sta     DLISTL
    mva     #4,sDLISTH              ; High byte of the display list.
    sta     DLISTH
    mva     #$0f,sCOLOR1            ; Full text luminance.
    mva     #rock_color_0,sCOLOR0   ; Team 0's rock color.
    mva     #outer_ring_color,sCOLPM0
    sta     sCOLPM1                 ; Set the players to blue for the outer ring.
    sta     sCOLPM2
    sta     sCOLPM3
    ldy     #$2d                    ; Build the ring pattern for the players,
    jsr     build_player_rings      ; at offset $2d.
    ldy     #$28                    ; Build the lines at offset $28.
    lda     #$82                    ; Use one pixel each from missiles 0 and 3.
    jsr     build_lines
    ldy     #$89                    ; Build the lines at $89 on the lower sheet.
    lda     #$82                    ; Use one pixel each from missiles 0 and 3.
    jsr     build_lines
    ldx     game_length
    cpx     #10
    bne     .no_score_cap           ; Don't need the missile endcap.
    ldy     #$76                    ; Put the end-line on the scoreboard at $76.
    lda     #$04                    ; Use one pixel from missile 1.
    ldx     #19                     ; Write 19 bytes.
.score_cap_loop
    sta     $1300,y
    iny
    dex
    bne     .score_cap_loop
    mva     #$0c,$1376              ; Make the top, center, and bottom of the
    sta     $1388                   ; end-line two pixels wide to extend the
    sta     $137f                   ; horizontal lines.
.no_score_cap
    mva     #1,current_end          ; Start on the first end.
    mva     #0,AUDCTL               ; Set master audio.
    mwa     crowd_sound,$fe         ; Play the crowd cheer.
    jsr     play_sound_1
    ; Fall through to the first end.

; Prepare the ice for a new end.
start_new_end subroutine
    mvx     #6,$ff                  ; Clear page six.  This is where the
    jsr     clear_page              ; free-guard info is stored.
    ldx     #$20                    ; Clear the ice memory.
.ice_clear_loop
    stx     $ff
    jsr     clear_page
    inx
    cpx     #$37                    ; Page $36 is the last page.
    bne     .ice_clear_loop
    ldx     #3                      ; Now build the hacks, 3 pixels tall.
    ldy     #0                      ; Index of 0.
    mwa     [$201d + bytes_per_row * 31],$fe
    mwa     [$201d + bytes_per_row * 40],$fc
.hack_loop
    lda     #$fc                    ; Build the hack in black.
    sta     ($fe),y                 ; Write to the top hack.
    sta     ($fc),y                 ; Write to the bottom hack.
    dex
    beq     .done_hack              ; Quit after 3 rows.
    lda     $fe                     ; Move to the next row for both hacks.
    sec
    sbc     #bytes_per_row
    bcs     .no_borrow_hack1
    dec     $ff
.no_borrow_hack1
    sta     $fe
    lda     $fc
    sec
    sbc     #bytes_per_row
    bcs     .no_borrow_hack2
    dec     $fd
.no_borrow_hack2
    sta     $fc
    jmp     .hack_loop              ; Do it again.
.done_hack
    ldy     #15                     ; Clear the various rock values.
    lda     #0
    sta     current_rock
.rock_clear_loop
    sta     rock_friction,y
    sta     rock_progress,y
    sta     rock_progress_high,y
    sta     rock_vertical,y
    dey
    bpl     .rock_clear_loop
    ldy     #rock_data_size * 16 - 1
.rock_data_clear_loop               ; Blank the main rock data.
    sta     rock_data_1,y
    dey
    cpy     #$ff
    bne     .rock_data_clear_loop
    ldy     #8                      ; Clear the old hammer.
    sty     $fb                     ; Need this for drawing, store it now.
.hammer_clear_loop
    dey
    bmi     .hammer_clear_done
    sta     $1577,y
    sta     $1580,y
    jmp     .hammer_clear_loop
.hammer_clear_done
    mwa     hammer_image,$fe        ; Set up the hammer image.
    mva     #$15,$fd
    lda     hammer_owner
    beq     .team_0_hammer
    lda     #$80
    jmp     .draw_hammer
.team_0_hammer
    lda     #$77
.draw_hammer
    sta     $fc
    jsr     copy_memory
    ldx     current_end
    cpx     #1                      ; If this is the first end, set
    bne     .not_first_end          ; the initial hammer position.
    mva     #61,hammer_hpos
    jmp     .hammer_done
.not_first_end
    cpx     game_length             ; If this is the extra end, don't
    beq     .advance_hammer         ; advance the hammer.
    bpl     .hammer_done
.advance_hammer
    lda     hammer_hpos
    clc
    adc     #16
    sta     hammer_hpos
.hammer_done
    mva     #$55,$fd                ; Draw the shot rocks at the bottom
    mwa     p0_status_base,$fe      ; of the screen.
    jsr     draw_8_status_rocks
    mva     #$aa,$fd
    mwa     p1_status_base,$fe
    jsr     draw_8_status_rocks
    ;jsr     apply_debug_data
    jsr     prepare_next_shot       ; Prepare the first rock.
    ; Fall through to the main game loop.

; Main game loop.
game_loop subroutine
    jsr     wait_for_vb
    lda     shot_mode               ; Check the shot mode.
    beq     .no_aim                 ; On 0, the shot is already in motion.
    jsr     handle_aim              ; Go do the aiming logic.
    jmp     .process_shooter
.no_aim
    lda     last_key_pressed        ; Check for pause.
    cmp     #$0d
    bne     .no_pause
    jmp     pause_loop
.no_pause
    mva     #$ff,last_key_pressed   ; Reset the key scan.
    dec     sweep_timer             ; Is it time to check the sweep?
    beq     .process_sweep
    jmp     .check_pebble
.process_sweep
    mva     #sweep_delay,sweep_timer
    mva     #0,sweep_value
    jmp     (sweep_value_vector)
sweep_reentry
    beq     .no_sweep               ; No change in sweep.
    mvy     #13,$ff
    jsr     divide                  ; Divide by 13 to get the visual.
    cpy     #0                      ; Play the sound if there's a visual.
    beq     .no_sweep_sound
    tya                             ; Save y, play_sound clobbers it.
    pha
    mwa     sweep_sound,$fe
    jsr     play_sound_4
    pla
    tay
.no_sweep_sound
    cpy     #9                      ; If the quotient is larger than 8,
    bpl     .big_sweep              ; split the display over players 0 and 3.
    sty     $ff                     ; Store the player 3 magnitude.
    mvy     #0,$fe                  ; Player 0 is blank.
    jmp     .draw_sweep
.big_sweep
    tya
    sec
    sbc     #8                      ; The value over 8 is the player 0 magnitude.
    sta     $fe
    mva     #8,$ff                  ; Player 3 magnitude is 8.
    jmp     .draw_sweep
.no_sweep
    mva     #0,$ff                  ; 0 magnitude for both players.
    sta     $fe
.draw_sweep
    lda     #0                      ; Start with a blank bit pattern.
    ldy     $ff                     ; Get the player 3 magnitude.
.p3_sweep_loop
    beq     .p3_sweep_done
    sec
    ror                             ; Rotate a bit in.
    inc     sweep_value             ; Increment the total sweep value.
    dey
    jmp     .p3_sweep_loop
.p3_sweep_done
    pha                             ; Save the player 3 pattern.
    lda     #0                      ; Start with a blank bit pattern.
    ldy     $fe                     ; Get the player 0 magnitude.
.p0_sweep_loop
    beq     .p0_sweep_done
    sec
    ror                             ; Rotate a bit in.
    inc     sweep_value             ; Increment the total sweep value.
    dey
    jmp     .p0_sweep_loop
.p0_sweep_done
    lsr     sweep_value             ; Cut the sweep value in half.
    sta     $ff                     ; Put the player 0 pattern in $ff.
    pla                             ; Get the player 3 pattern back.
    tay                             ; Put a copy in y.
    ldx     #4                      ; 5 bytes drawn on each player.
.sweep_draw_loop
    tya                             ; Get the player 3 pattern.
    sta     $17ca,x                 ; Write it.
    lda     $ff                     ; Get the player 0 pattern.
    sta     $14ca,x                 ; Write it.
    dex
    bpl     .sweep_draw_loop
.check_pebble
    ldy     pebble_frames           ; Update the pebble friction if necessary.
    iny
    cpy     #60
    beq     .increase_pebble
    sty     pebble_frames
    jmp     .process_shooter
.increase_pebble
    mvy     #0,pebble_frames
    inc     pebble_friction
.process_shooter
    jsr     get_rock_number         ; Save the shooter's horizontal
    lda     rock_progress,x         ; position so we can later check
    pha                             ; for scrolling.
    lda     rock_progress_high,x
    pha
    lda     rock_vertical,x         ; Save the shooter's vertical position
    pha                             ; to allow for curler movement.
    ldy     #0                      ; Index into the moving rock list.
.move_rocks_loop
    ldx     rock_list,y             ; Load the next rock to be moved.
    beq     .done_move_rocks        ; The list is terminated by 0.
    tya                             ; Save y, it will get clobbered
    pha                             ; during this process.
    jsr     process_rock            ; Go handle rock movement and interactions.
    beq     .rock_removed
    pla
    tay
    cpy     #15                     ; After 16 rocks, stop.
    beq     .done_move_rocks
    iny
    jmp     .move_rocks_loop
.rock_removed
    pla
    tay
    jmp     .move_rocks_loop        ; Run the loop again with the same index.
.done_move_rocks
    cpy     #0                      ; If y is still 0, and the shot mode
    bne     .check_blink            ; is shot_mode_none, the shot is complete.
    lda     shot_mode
    bne     .check_blink
    jsr     stop_sound_1            ; Kill the "roar" sound.
    mva     #cpu_sweep_mode_none,cpu_sweep_mode
    dec     shot_end_delay          ; Check the delay timer.
    bne     .check_blink
    jsr     prepare_next_shot
.check_blink
    dec     blink_timer             ; Check the status area blinker.
    bne     .done_status_blink
    jsr     blink_status_rock
.done_status_blink
    ldx     current_rock_number     ; See if the shooter has moved vertically,
    pla                             ; and if so, scroll the curler.
    tay
    lda     shot_mode
    cmp     #shot_mode_curl
    bne     .done_vertical          ; Shot already released, no need to handle.
    tya
    sec
    sbc     rock_vertical,x         ; See if the rock moved.
    beq     .done_vertical          ; No movement.
    bpl     .move_curler_up         ; Positive is moving up.
    tya                             ; Get the original value back.
.move_curler_down_loop
    sta     $ff                     ; Save the old position.
    jsr     move_curler_down
    ldx     current_rock_number
    lda     $ff
    clc
    adc     #1
    cmp     rock_vertical,x
    beq     .done_vertical
    jmp     .move_curler_down_loop
.move_curler_up
    tya                             ; Get the original value back.
.move_curler_up_loop
    sta     $ff                     ; Save the old position.
    jsr     move_curler_up
    ldx     current_rock_number
    lda     $ff
    sec
    sbc     #1
    cmp     rock_vertical,x
    bne     .move_curler_up_loop
.done_vertical
    pla                             ; See if the shooter has moved to the right,
    cmp     rock_progress_high,x    ; and if so, do scrolling.
    bcc     .no_scroll
    pla
    cmp     rock_progress,x
    bcs     .done_scroll
    lda     shot_mode               ; Apply curl after the shot has been released.
    bne     .no_curl     
    ldx     current_rock
    jsr     apply_curl              ; The rock is still moving, so apply curl.
.no_curl
    ldx     current_rock_number
    jsr     perform_scroll
    jmp     .done_scroll
.no_scroll
    pla                             ; Remove the high byte from the stack.
.done_scroll
    jmp     game_loop

; Pause handling loop
pause_loop subroutine
    mwa     $1000,$fe               ; Save the old text.
    mwa     $f00,$fc
    mva     #40,$fb
    jsr     copy_memory
    lda     $1000
    cmp     #$23                    ; If the first character in the display
    bne     .player_pause           ; is a C, the CPU is shooting.
    mwa     cpu_paused_text,$fc
    jmp     .place_pause_text
.player_pause
    mwa     paused_text,$fc
.place_pause_text
    jsr     copy_text_full          ; Place the pause text.
    jsr     stop_all_sounds
.loop
    mva     #$ff,last_key_pressed   ; Reset the key scan.
    jsr     wait_for_vb
    lda     last_key_pressed
    cmp     #$0e
    bne     .no_reset
    jmp     start_title
.no_reset
    cmp     #$0d                    ; Check for unpause.
    bne     .loop
    mwa     $1000,$fc               ; Restore the old text.
    mwa     $f00,$fe
    mva     #40,$fb
    jsr     copy_memory
    mva     #$ff,last_key_pressed
    mwa     roar_sound,$fe
    jsr     play_sound_1
    ldy     sweeper_vpot_1          ; Reset the sweeper's sticks.
    lda     POT0,y
    sta     last_stick
    ldy     sweeper_vpot_2
    lda     POT0,y
    sta     last_stick_2
    mva     #$ff,last_key_pressed   ; Reset the key scan.
    jmp     game_loop

; Sweeping subroutines.
one_stick_sweep subroutine
    ldy     sweeper_vpot_1          ; Get the sweeper's pot.
    lda     POT0,y                  ; Calculate the sweeping.
    cmp     last_stick              ; Figure out which value is larger.
    bcc     .sweep_up
    mvy     last_stick,$ff          ; Put the smaller value in $ff.
    sta     last_stick              ; Update the last stick.
    jmp     .calculate_sweep
.sweep_up
    ldy     last_stick              ; Get the larger value.
    sta     last_stick              ; Update the last stick.
    sta     $ff                     ; Put the smaller value in $ff.
    tya                             ; Larger value in the accumulator.
.calculate_sweep
    sec
    sbc     $ff                     ; Get the difference in vertical.
    jmp     sweep_reentry

two_stick_sweep subroutine
    ldy     sweeper_vpot_1          ; Get the first sweeper's pot.
    lda     POT0,y                  ; Calculate the sweeping.
    cmp     last_stick              ; Figure out which value is larger.
    bcc     .sweep_up
    mvy     last_stick,$ff          ; Put the smaller value in $ff.
    sta     last_stick              ; Update the last stick.
    jmp     .calculate_sweep
.sweep_up
    ldy     last_stick              ; Get the larger value.
    sta     last_stick              ; Update the last stick.
    sta     $ff                     ; Put the smaller value in $ff.
    tya                             ; Larger value in the accumulator.
.calculate_sweep
    sec
    sbc     $ff                     ; Get the difference in vertical.
    sta     $fe                     ; Save the first value.
    ldy     sweeper_vpot_2          ; Get the second sweeper's pot.
    lda     POT0,y                  ; Calculate the sweeping.
    cmp     last_stick_2            ; Figure out which value is larger.
    bcc     .sweep_up2
    mvy     last_stick_2,$ff        ; Put the smaller value in $ff.
    sta     last_stick_2            ; Update the last stick.
    jmp     .calculate_sweep2
.sweep_up2
    ldy     last_stick_2            ; Get the larger value.
    sta     last_stick_2            ; Update the last stick.
    sta     $ff                     ; Put the smaller value in $ff.
    tya                             ; Larger value in the accumulator.
.calculate_sweep2
    sec
    sbc     $ff                     ; Get the difference in vertical.
    clc
    adc     $fe                     ; Add the first value.
    ror                             ; Divide by 2, bring in the carry bit.
    jmp     sweep_reentry

; Calculates the current rock number from the active rock pointer.
; Result is placed in x.
get_rock_number subroutine
    lda     #rock_data_1            ; Start from the rock base.
    ldx     #0                      ; Assume rock 0.
    clc
.loop
    cmp     current_rock
    bne     .next_rock
    rts
.next_rock
    adc     #rock_data_size
    inx
    jmp     .loop

; Calculate the number of the rock whose address is in x.
; Places the number in y.
get_rock_number_from_x subroutine
    lda     #rock_data_1            ; Start from the rock base.
    stx     $ff                     ; Put the address somewhere handy.
    ldy     #0                      ; Assume rock 0.
    clc
.loop
    cmp     $ff
    bne     .next_rock
    rts
.next_rock
    adc     #rock_data_size
    iny
    jmp     .loop

; Convert the rock number in x to a rock pointer.
get_rock_pointer_from_x subroutine
    lda     #rock_data_1            ; Start from the rock base.
    clc
.loop
    dex
    bmi     .done                   ; Loop done, bail out.
    adc     #rock_data_size
    jmp     .loop
.done
    tax                             ; Put the address in x.
    rts

; Handle the shot aiming by the player.  Expects the
; current shot mode to be in the accumulator.
handle_aim subroutine
    cmp     #shot_mode_aim          ; Check for the aim mode.
    beq     .do_aim
    jmp     .no_aim
.do_aim
    ldx     last_key_pressed
    cpx     #$0e
    bne     .no_reset_on_aim
    ldx     #$ff                    ; Clear the stack to erase the jsr.
    txs
    jmp     start_title
.no_reset_on_aim
    mvx     #$ff,last_key_pressed
    jsr     erase_aim_cursors
    jmp     (aim_value_vector)      ; Figure out where to draw the new cursor.
aim_reentry
    cmp     #aim_lower_bound        ; Clamp to the lower boundary.
    bcs     .aim_lower_bound_ok
    lda     #aim_lower_bound
    jmp     .adjust_aim
.aim_lower_bound_ok
    cmp     #aim_upper_bound        ; Clamp to the upper boundary.
    bcc     .adjust_aim
    lda     #aim_upper_bound
.adjust_aim
    pha                             ; Save the actual value for later.
    sec
    sbc     last_stick              ; Figure out which direction to go.
    beq     .no_aim_adjust
    bcc     .move_aim_up            ; New value is less, move up.
    tax                             ; Counter for the rows to move.
.aim_down_loop
    lda     aim_cursor              ; Get the cursor address.
    clc
    adc     #bytes_per_row
    sta     aim_cursor
    lda     aim_cursor + 1
    adc     #0
    sta     aim_cursor + 1
    lda     aim_cursor_far          ; Move the far cursor thrice.
    clc
    adc     #bytes_per_row * 3
    sta     aim_cursor_far
    lda     aim_cursor_far + 1
    adc     #0
    sta     aim_cursor_far + 1
    dex
    beq     .no_aim_adjust
    jmp     .aim_down_loop
.move_aim_up
    tax                             ; Counter for the rows to move.
.aim_up_loop
    lda     aim_cursor              ; Get the cursor address.
    sec
    sbc     #bytes_per_row
    sta     aim_cursor
    lda     aim_cursor + 1
    sbc     #0
    sta     aim_cursor + 1
    lda     aim_cursor_far          ; Move the far cursor thrice.
    sec
    sbc     #bytes_per_row * 3
    sta     aim_cursor_far
    lda     aim_cursor_far + 1
    sbc     #0
    sta     aim_cursor_far + 1
    inx
    bne     .aim_up_loop
.no_aim_adjust
    pla
    sta     last_stick              ; Update the joystick position.
    ldx     current_rock
    ;lda     #96                     ; TEST CODE 96 is straight on
    sta     rock_v_speed,x          ; Save the value in the rock for later.
    jsr     far_aim_arrow_visible   ; Check for the far arrow.
    sta     $f0                     ; Save the result.
    mva     aim_cursor,$fe          ; Reset the cursor addresses.
    mva     aim_cursor + 1,$ff
    mva     aim_cursor_far,$fc
    mva     aim_cursor_far + 1,$fd
    ldx     #4                      ; Draw the new cursors, 5 pixels high.
    ldy     #0
.aim_draw_loop
    lda     aim_arrow,x
    sta     ($fe),y
    pha                             ; Save the pattern.
    lda     $f0                     ; Check the far arrow.
    bne     .no_far_arrow
    pla                             ; Get the pattern back.
    sta     ($fc),y
    jmp     .update_aim_addresses
.no_far_arrow
    pla                             ; Fix the stack.
.update_aim_addresses
    lda     $fe
    clc
    adc     #bytes_per_row
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    lda     $fc
    clc
    adc     #bytes_per_row
    sta     $fc
    lda     $fd
    adc     #0
    sta     $fd
    dex
    bpl     .aim_draw_loop
    jmp     .aim_done
.no_aim
    cmp     #shot_mode_weight       ; See if the shooter is setting weight.
    beq     .set_weight
    jmp     .no_weight
.set_weight
    jmp     (key_value_vector)      ; Check for a mode change.
key_reentry
    bmi     .no_weight_mode_change  ; No key press.
    beq     .abort_weight_mode      ; On 0, abort and go back to aim.
    cmp     #$0e
    bne     .no_reset_on_weight
    ldx     #$ff                    ; Reset the stack to clear the jsr.
    txs
    jmp     start_title
.no_reset_on_weight
    cmp     weight_mode             ; Compare against the existing weight mode.
    beq     .no_weight_mode_change  ; Nothing changed.
    cmp     #weight_mode_takeout + 1
    bpl     .no_weight_mode_change  ; Key out of range.
    sta     weight_mode             ; Update the weight mode.
    jsr     place_weight_mode_text  ; Update the weight text.
    mva     #$ff,last_key_pressed   ; Reset the key.
    jmp     .no_weight_mode_change
.abort_weight_mode
    jsr     erase_weight_gauge
    mva     #shot_mode_aim,shot_mode
    rts
.no_weight_mode_change
    jmp     (weight_value_vector)   ; Get the stick position.
weight_reentry
    cmp     #weight_lower_bound     ; Check the lower bound.
    bcs     .weight_lower_bound_ok
    lda     #weight_lower_bound
    jmp     .adjust_weight
.weight_lower_bound_ok
    cmp     #weight_upper_bound     ; Check the upper bound.
    bcc     .adjust_weight
    lda     #weight_upper_bound
.adjust_weight
    sec
    sbc     #weight_lower_bound     ; Subtract the lower bound to get the weight.
    ldx     current_rock
    sta     rock_h_speed,x          ; Stash it the rock for later processing.
    lsr                             ; Divide by two for visual purposes.
    sta     $fb                     ; Save it for later.
    mwa     [weight_meter_left + 1],$fc
    ldy     #0
.weight_meter_loop_outer
    ldx     #6                      ; Weight scale is 6 high.
    mva     $fc,$fe                 ; Prepare the address to draw from.
    mva     $fd,$ff
    lda     $fb                     ; Get the weight back.
    cmp     #4                      ; If it's 4 or more, draw a full byte.
    bpl     .draw_full_weight_byte
    cmp     #3                      ; 6 bits.
    bne     .not_3_weight
    lda     #$a8
    jmp     .draw_weight_byte
.not_3_weight
    cmp     #2
    bne     .not_2_weight
    lda     #$a0                    ; 4 bits.
    jmp     .draw_weight_byte
.not_2_weight
    cmp     #1
    bne     .blank_weight_byte
    lda     #$80                    ; 2 bits.
    jmp     .draw_weight_byte
.blank_weight_byte
    lda     #0                      ; 0 bits, blank section.
    jmp     .draw_weight_byte
.draw_full_weight_byte
    lda     #$aa
.draw_weight_byte
    sta     ($fe),y                 ; Write the byte.
    dex
    bne     .update_weight_address
    lda     $fc                     ; On the fourth byte, drawing is done.
    cmp     [<#weight_meter_left] + 4
    bne     .draw_weight_continue
    jmp     .aim_done
.draw_weight_continue
    lda     $fb                     ; Get the weight back.
    sec
    sbc     #4                      ; Drop it by four.
    sta     $fb                     ; Update the remaining weight.
    inc     $fc                     ; Increment the starting address.
    jmp     .weight_meter_loop_outer
.update_weight_address
    pha                             ; Save the bit pattern.
    lda     $fe                     ; Advance to the next row.
    clc
    adc     #bytes_per_row
    bcc     .no_carry_weight_draw
    inc     $ff
.no_carry_weight_draw
    sta     $fe
    pla                             ; Get the bit pattern back.
    jmp     .draw_weight_byte
.no_weight
    mva     #$ff,last_key_pressed   ; Ignore keys.
    mwa     curl_arrow_base,$fe     ; Store the drawing address.
    jmp     (curl_value_vector)     ; Get the stick position.
curl_reentry
    ldx     current_rock
    cmp     #80                     ; Check the low value.
    bcc     .curl_up
    cmp     #120                    ; Check the high value.
    bcs     .curl_down
    lda     #0
    sta     rock_r_speed,x
    mwa     curl_flat,$fc           ; Store the source image, no curl.
    jmp     .draw_curl_direction
.curl_up
    lda     #-1
    sta     rock_r_speed,x
    mwa     curl_arrow_up,$fc       ; Store the source image, arrow up.
    jmp     .draw_curl_direction
.curl_down
    lda     #1
    sta     rock_r_speed,x
    mwa     curl_arrow_down,$fc     ; Store the source image, arrow down.
.draw_curl_direction
    ldx     #14                     ; 14 rows to draw.
.draw_curl_loop
    ldy     #0                      ; Graphics index 0.
    lda     ($fc),y                 ; Get the image byte.
    sta     ($fe),y                 ; Write it.
    iny                             ; Step forward.
    lda     ($fc),y
    sta     ($fe),y
    dex                             ; Check the row count.
    beq     .aim_done
    lda     $fe                     ; Update the target addresses.
    clc
    adc     #bytes_per_row
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    lda     $fc
    clc
    adc     #2
    sta     $fc
    lda     $fd
    adc     #0
    sta     $fd
    jmp     .draw_curl_loop
.aim_done
    lda     trigger_timer           ; Check the trigger debounce.
    beq     .process_trigger
    dec     trigger_timer
    jmp     .no_trigger_change
.process_trigger
    jmp     (aim_trig_vector)       ; Check the trigger for confirmation.
trig_reentry
    cmp     trigger_buffer          ; Only honor the trigger if it changed.
    bne     .trigger_change
    jmp     .no_trigger_change
.trigger_change
    cmp     #0
    beq     .no_trigger_release
    jmp     .trigger_release        ; On 1, the button was released.
.no_trigger_release
    inc     shot_mode               ; Go to the next shot mode.
    pha                             ; Save the trigger state.
    mva     #trigger_debounce,trigger_timer
    lda     shot_mode               ; Prep for the next shot mode.
    cmp     #shot_mode_weight
    beq     .prepare_weight_mode
    cmp     #shot_mode_curl
    beq     .prepare_curl_mode
    jmp     .release_shot
.prepare_weight_mode
    mva     #1,AUDCTL               ; AUDCTL 0 was used for the crowd.
    lda     #weight_mode_guard      ; Prepare the weight mode.
    sta     weight_mode
    jsr     place_weight_mode_text
    mva     #$ff,last_key_pressed   ; Reset the key tracking.
    ldy     #0                      ; Draw the weight scale, 6 high.
    ldx     #6
    mwa     weight_meter_left,$fe   ; Left edge of the scale.
    mwa     weight_meter_right,$fc  ; Right edge of the scale.
.weight_scale_loop
    lda     #$03                    ; Two pixels on the right edge.
    sta     ($fe),y
    lda     #$c0                    ; Two pixels on the left edge.
    sta     ($fc),y
    dex
    bne     .continue_weight
    jmp     .finish_prepare
.continue_weight
    lda     $fe                     ; Update the addresses.
    clc
    adc     #bytes_per_row
    bcc     .no_carry_weight_loop1
    inc     $ff
.no_carry_weight_loop1
    sta     $fe
    lda     $fc
    clc
    adc     #bytes_per_row
    bcc     .no_carry_weight_loop2
    inc     $fd
.no_carry_weight_loop2
    sta     $fc
    jmp     .weight_scale_loop
.prepare_curl_mode
    ldx     current_rock            ; Evaluate the aim and weight.
    lda     weight_mode             ; Find the divisor and base value
    cmp     #weight_mode_guard      ; for the current shot mode.
    bne     .not_guard_weight
    mva     #guard_ratio,$ff
    mva     #guard_low,$fe
    jmp     .divide_weight
.not_guard_weight
    cmp     #weight_mode_draw
    bne     .takeout_weight
    mva     #draw_ratio,$ff
    mva     #draw_low,$fe
    jmp     .divide_weight
.takeout_weight
    mva     #takeout_ratio,$ff
    mva     #takeout_low,$fe
.divide_weight
    lda     rock_h_speed,x          ; Divide the speed from the gauge
    jsr     divide                  ; by the ratio value.
    tya
    clc                             ; Add the base weight.
    adc     $fe
    ;lda     #79                     ; TEST CODE weight at shot is 46-100
    sta     rock_h_speed,x          ; Store the final weight value.
    jsr     get_handedness
    bne     .left_handed
    ldx     current_rock
    lda     rock_v_speed,x          ; Get the aim value.
    sec
    sbc     #aim_lower_bound + 18   ; Normalize it, and less than 18 is shooting up.
    bne     .calculate_aim
    jmp     .store_aim              ; Aim of 0, can't do division!
.left_handed
    ldx     current_rock
    lda     rock_v_speed,x          ; Get the aim value.
    sec
    sbc     #aim_lower_bound + 31   ; Normalize it, and less than 31 is shooting up.
    bne     .calculate_aim
    jmp     .store_aim              ; Aim of 0, can't do division!
.calculate_aim
    pha                             ; Save the value so the sign can be checked.
    jsr     absolute_value
    sta     $f0                     ; Save the positive value.
    lda     rock_h_speed,x          ; Need to do some math.
    sta     $fc
    mva     #0,$fd                  ; Aim distance * 100 / h_speed = frames to the aim line.
    mwa     [#aim_distance * 100],$fa
    jsr     divide_16
    mva     $fe,$fc                 ; Move the result to the divisor position.
    mva     $ff,$fd
    lda     $f0                     ; Multiply the vertical by 250.
    mvy     #250,$ff
    jsr     multiply
    sta     $fa                     ; Put the vertical distance in the dividend position.
    stx     $fb
    jsr     divide_16               ; This quotient will be the vertical speed.
    ldx     current_rock            ; Retrieve the rock pointer.
    pla                             ; Get the original value back.
    bmi     .no_direction_bit       ; If positive, need to set the direction bit.
    lda     rock_flags,x            ; Set the direction bit to indicate downward vertical.
    ora     #rock_v_direction
    sta     rock_flags,x
.no_direction_bit
    lda     $fe                     ; Result should be one byte.
.store_aim
    sta     rock_v_speed,x          ; Write the final speed.
    lda     #0
    sta     rock_r_time,x
    jsr     activate_rock
    mwa     roar_sound,$fe
    jsr     play_sound_1
    jsr     erase_aim_cursors
    jsr     erase_aim_scales
    jsr     erase_weight_gauge
    jmp     .finish_prepare
.release_shot
    ldy     sweeper_vpot_1          ; Establish a baseline for sweeping.
    lda     POT0,y
    sta     last_stick
    ldy     sweeper_vpot_2
    lda     POT0,y
    sta     last_stick_2
    mva     #shot_mode_none,shot_mode
    mva     #4,prior_low            ; Playfield priority over player/missiles.
    ldx     current_rock
    lda     rock_r_speed,x          ; Get the curl speed.
    beq     .finish_release         ; On 0, there is, as they say, no handle.
    bmi     .no_curl_flag           ; On negative, going up.
    lda     rock_flags,x            ; Set the direction flag for down.
    ora     #rock_rotation
    sta     rock_flags,x
.no_curl_flag
    lda     #48                     ; Base curl value is 48.
    sta     rock_r_speed,x
    sta     rock_r_time,x
.finish_release
    jsr     erase_curl_arrow        ; Erase the curl arrow.
.finish_prepare
    pla                             ; Retrieve the trigger state.
.trigger_release
    sta     trigger_buffer          ; Update the trigger buffer.
.no_trigger_change
    rts

; Routine for getting a real player's vertical aim value.
get_player_aim subroutine
    ldy     shooter_vpot            ; Get the pot offset.
    lda     POT0,y
    jmp     aim_reentry

; Routine for getting a real player's trigger value.
get_player_trig
    ldy     shooter_trig            ; Get the trig offset.
    lda     TRIG0,y
    jmp     trig_reentry

; Routine for getting a real player's horizontal weight value.
get_player_weight subroutine
    ldy     shooter_hpot            ; Get the pot offset.
    lda     POT0,y
    jmp     weight_reentry

; Routine for getting the player's vertical curl value.
get_player_curl subroutine
    ldy     shooter_vpot            ; Get the pot offset.
    lda     POT0,y
    jmp     curl_reentry

; Routine for reading the player's keypad.
get_player_keypress subroutine
    lda     last_key_pressed
    jmp     key_reentry

; Draw the two aim assist scales.
draw_aim_scales subroutine
    ldx     #71                     ; 70 rows high, plus a hacky skip.
    mwa     aim_scale_near,$fe      ; Address of the nearer scale.
    mva     #7,$fb                  ; Change the pattern every 5 lines.
    mva     #5,$fc                  ; Need to skip a line after 5 ticks.
    lda     #$fc                    ; Pattern to write.
.loop
    ldy     #0                      ; 0 offset.
    sta     ($fe),y
    ldy     #aim_scale_far - #aim_scale_near
    sta     ($fe),y                 ; Offset to the far scale.
    dex
    beq     .done
    dec     $fb                     ; Drop the counter.
    beq     .full_line              ; At 0, display a big line.
    lda     #$c0                    ; Thin line.
    pha                             ; Stack it.
    jmp     .update_address
.full_line
    mva     #7,$fb                  ; Reset the counter.
    dec     $fc                     ; See if this is the skip row.
    beq     .skip_row
    lda     #$fc                    ; Load the big line.
    pha                             ; Stack it.
    jmp     .update_address
.skip_row
    lda     #$fc                    ; Load the row pattern.
    jmp     .loop
.update_address
    lda     $fe
    clc
    adc     #bytes_per_row
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    pla                             ; Get the pattern.
    jmp     .loop
.done
    rts

; Erase the two aim assist scales.
erase_aim_scales subroutine
    ldx     #70                     ; 70 rows high.
    mwa     aim_scale_near,$fe      ; Address of the nearer scale.
.loop
    lda     #0                      ; Write 0s.
    tay                             ; 0 offset.
    sta     ($fe),y                 ; Clear the nearer scale.
    ldy     #aim_scale_far - #aim_scale_near
    sta     ($fe),y                 ; Clear the farther scale.
    dex
    beq     .done
    lda     $fe                     ; Update the base address.
    clc
    adc     #bytes_per_row
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    jmp     .loop
.done
    rts

; Reset the playfield pointers in the two display lists.
reset_scroll subroutine
    jsr     wait_for_vb
    mvy     #$0f,current_scroll     ; Reset the fine scroll.
    sty     HSCROL
    mvy     #0,scroll_list_offset   ; Reset the scroll pointer.
    ldx     #70                     ; 70 rows to update.
    mwa     $4e1,$fe                ; Address of the first memory scan.
    mwa     $8e1,$fc                ; Address of the second memory scan.
    mwa     $201d,$fa               ; Starting address for list 1.
    mwa     $2019,$f8               ; Starting address for list 2.
.loop
    lda     $fa
    sta     ($fe),y
    lda     $f8
    sta     ($fc),y
    iny
    lda     $fb
    sta     ($fe),y
    lda     $f9
    sta     ($fc),y
    dex
    beq     .done
    iny                             
    iny                             ; Hop over the mode byte.
    lda     $fa
    clc
    adc     #bytes_per_row
    bcc     .no_carry_list_1
    inc     $fb
.no_carry_list_1
    sta     $fa
    lda     $f8
    clc
    adc     #bytes_per_row
    bcc     .no_carry_list_2
    inc     $f9
.no_carry_list_2
    sta     $f8
    jmp     .loop
.done
    mva     #$4,sDLISTH             ; Go back to the first display list.
    sta     DLISTH
    rts

; Perform all scrolling logic, updating display pointers and
; swapping the display lists if necessary.  Expects the current
; rock number in x.
perform_scroll subroutine
    ldy     rock_progress,x
    cpy     #80                     ; Don't scroll before 80 pixels.
    beq     .left_bound_fail
    bcs     .left_bound_ok
.left_bound_fail
    lda     shot_mode               ; If the rock is still being
    beq     .shot_released          ; held, move the curler to the right.
    cpy     #8                      ; After 8 pixels, scroll everything.
    bcs     .scroll_all_players
    cpy     #4
    bcc     .scroll_upper_body      ; Don't scroll the pants at first.
    bne     .done_body_update
    mva     #1,sizep2_low           ; Go to a double size body.
    jsr     get_handedness          ; Figure out the particulars of the update.
    sta     $ff                     ; Save the handedness for indexing.
    bne     .left_handed
    ldy     #0                      ; Starting read index.
    jmp     .prepare_draw
.left_handed
    ldy     #6                      ; Starting read index.
.prepare_draw
    ldx     curler_p2_address       ; Get the base address for drawing.
.draw_loop
    lda     curler_p2_extended,y
    sta     $1600,x
    lda     $ff                     ; Adjust y based on handedness.
    bne     .left_handed_index
    cpy     #6
    beq     .done_body_update
    iny
    jmp     .continue_loop
.left_handed_index
    dey
    bmi     .done_body_update
.continue_loop
    inx
    jmp     .draw_loop
.done_body_update
    rts
.scroll_all_players
    inc     p3_low_h
.scroll_upper_body
    inc     p1_low_h
    inc     p2_low_h
.shot_released
    rts
.left_bound_ok
    cpy     #252                    ; Don't scroll after 252.
    bcc     .right_bound_ok
    rts
.right_bound_ok
    cpy     #202                    ; Don't scroll the backline too early.
    bcc     .no_shift_m3
    dec     m3_low_h
.no_shift_m3
    ldy     m0_low_h
    cpy     #40
    bcs     .no_shift_m0
    ldy     #$8e
    jsr     build_player_rings      ; Create the player rings on the bottom.
    mva     #0,sizep2_low           ; Go back to normal size.
    jsr     create_sweeping_gauge   ; Redraw the sweep gauge.
    mva     #255,p0_low_h           ; Position the rings offscreen to the right.
    mva     #7,p1_low_h
    mva     #15,p2_low_h
    mva     #23,p3_low_h
    mva     #outer_ring_color,pm1_low_color_1
    sta     pm2_low_color_1
    sta     pm3_low_color
    sta     pm1_low_color_3
    sta     pm2_low_color_3
    mva     #inner_ring_color,pm1_low_color_2
    sta     pm2_low_color_2
    ldy     #218                    ; Reposition m0 to the other hogline.
.no_shift_m0
    dey
    sty     m0_low_h                ; Scroll the players and missiles.
    lda     m1_low_h                ; Don't scroll the missiles past 0,
    beq     .no_scroll_m1           ; or they'll wrap around into the
    dec     m1_low_h                ; rings in the far end.
.no_scroll_m1
    lda     m2_low_h
    beq     .no_scroll_m2
    dec     m2_low_h
.no_scroll_m2
    dec     p0_low_h
    dec     p1_low_h
    dec     p2_low_h
    dec     p3_low_h
    ldy     current_scroll          ; Test the scroll to see how many
    cpy     #$a                     ; pointers to update this frame.
    bmi     .short_frame
    ldx     #5                      ; Update 5 pointers.
    jmp     .update_pointers
.short_frame
    ldx     #4                      ; Update 4 pointers.
.update_pointers
    lda     sDLISTH                 ; Get the current_display list.
    eor     #$c                     ; Flip the 4 and 8 bits.
    sta     $ff
    mva     #$e1,$fe                ; The base pointer address.
    ldy     scroll_list_offset      ; Get the offset to start from.
.update_loop
    lda     ($fe),y                 ; Get the low byte.
    clc                             ; The list being updated is 4 bytes
    adc     #8                      ; behind, so add 8 to get the new location.
    bcc     .no_carry
    sta     $fd
    iny
    lda     ($fe),y
    adc     #0                      ; Carry bit is set, this increments.
    sta     ($fe),y
    dey
    lda     $fd
.no_carry
    sta     ($fe),y
    iny
    iny
    iny                             ; Add 3 to y to reach the next pointer.
    dex
    bne     .update_loop
    ldx     current_scroll          ; Retest the current scroll to see if
    beq     .flip_lists             ; the display lists need to flip.
    sty     scroll_list_offset      ; No flip.  Update the offset and
    dex                             ; adjust the fine scroll.
    stx     current_scroll
    stx     HSCROL
    rts
.flip_lists
    mvy     #$f,current_scroll      ; Reset the fine scroll.
    sty     HSCROL
    mvy     #0,scroll_list_offset   ; Reset the offset.
    lda     sDLISTH                 ; Flip the high byte of the display list.
    eor     #$c                     ; Toggle the 4 and 8 bits.
    sta     sDLISTH
    sta     DLISTH
    rts

; Erase the shot cursor used during aim mode.
erase_aim_cursors subroutine
    jsr     far_aim_arrow_visible   ; Check for the far arrow.
    sta     $f0                     ; Save the result.
    mva     aim_cursor,$fe          ; Ready the cursor addresses.
    mva     aim_cursor + 1,$ff
    mva     aim_cursor_far,$fc
    mva     aim_cursor_far + 1,$fd
    ldx     #4                      ; 5 pixels high.
    ldy     #0
.loop
    tya
    sta     ($fe),y                 ; Blank the locations.
    lda     $f0
    bne     .no_far_arrow
    sta     ($fc),y
.no_far_arrow
    lda     $fe                     ; Go to the next row.
    clc
    adc     #bytes_per_row
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    lda     $fc
    clc
    adc     #bytes_per_row
    sta     $fc
    lda     $fd
    adc     #0
    sta     $fd
    dex
    bpl     .loop
    rts

; Check to see if the far aim arrow is in range.
; Places 0 in the accumulator if true, 1 if not.
far_aim_arrow_visible subroutine
    jsr     get_handedness
    beq     .right_handed
    lda     last_stick
    cmp     #far_lower_bound_l
    bcc     .not_visible
    cmp     #far_upper_bound_l
    beq     .visible
    bcc     .visible
    jmp     .not_visible
.right_handed
    lda     last_stick
    cmp     #far_lower_bound_r
    bcc     .not_visible
    cmp     #far_upper_bound_r
    beq     .visible
    bcc     .visible
.not_visible
    lda     #1
    rts
.visible
    lda     #0
    rts

; Erase the curl arrow.
erase_curl_arrow subroutine
    mwa     curl_arrow_base,$fe
    ldx     #14                     ; 14 rows to erase.
.loop
    ldy     #0                      ; Index 0.
    tya                             ; Write 0s.
    sta     ($fe),y
    iny                             ; Hit the next byte.
    sta     ($fe),y
    dex                             ; Check the row count.
    beq     .done
    lda     $fe                     ; Update the target address.
    clc
    adc     #bytes_per_row
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    jmp     .loop
.done
    rts

; Handles the blinking of the lower status rocks.
blink_status_rock subroutine
    mva     #blink_length,blink_timer
    jsr     get_rock_number         ; Get the number of the rock to blink.
    txa
    and     #1                      ; Even rocks are player 0.
    beq     .p0_blink
    mwa     p1_status_base,$fe      ; Set the status base address.
    mva     #$aa,$fd                ; Color pattern for player 1.
    jmp     .find_rock
.p0_blink
    mwa     p0_status_base,$fe      ; Set the status base address.
    mva     #$55,$fd                ; Color pattern for player 0.
.find_rock
    txa                             ; Divide the rock number by 2
    lsr                             ; to get the individual player rock.
    tax
.loop
    cpx     #0                      ; Find the specific rock.
    beq     .blink_rock
    clc
    lda     $fe                     ; Advance two bytes.
    adc     #2
    bcc     .no_blink_carry
    inc     $ff
.no_blink_carry
    sta     $fe
    dex
    jmp     .loop
.blink_rock
    ldy     #0
    lda     ($fe),y                 ; Check for existing data.
    beq     .blink_on
    jsr     erase_status_rock
    rts
.blink_on
    jsr     draw_status_rock
    rts

; Build the ring patterns for the players at a given offset in y.
build_player_rings subroutine
    ldx     #0                      ; Index into the graphics image.
.loop1
    lda     player0_rings,x
    sta     $1400,y
    lda     player1_rings,x
    sta     $1500,y
    lda     player2_rings,x
    sta     $1600,y
    lda     player3_rings,x
    sta     $1700,y
    inx
    iny
    cpx     #30                     ; After 30 rows, reverse direction.
    bne     .loop1
    dex
.loop2
    lda     player0_rings,x         ; Now mirror the top half.
    sta     $1400,y
    lda     player1_rings,x
    sta     $1500,y
    lda     player2_rings,x
    sta     $1600,y
    lda     player3_rings,x
    sta     $1700,y
    iny
    dex
    bpl     .loop2
    rts

; Build lines using the pattern in the accumulator at the offset in y.
build_lines subroutine
    ldx     #70                     ; Sheet is 70 pixels high.
.loop
    sta     $1300,y
    iny
    dex
    bne     .loop
    rts

; Create the sweeping gauge at the bottom of players 0 and 3.
create_sweeping_gauge subroutine
    ldy     #10                     ; Eleven rows.
.p3_loop
    lda     sweeping_label_3,y
    sta     $17c4,y
    dey
    bpl     .p3_loop
    ldy     #8                      ; Nine rows.
.p0_loop
    lda     sweeping_label_0,y
    sta     $14c6,y
    dey
    bpl     .p0_loop
    rts

; Ready the next shot.
prepare_next_shot subroutine
    lda     current_rock            ; Check the current rock address.
    bne     .advance_rock           ; If it's zero, this is the first rock.
    lda     hammer_owner            ; See who gets first shot.
    beq     .team_1_first
    lda     #rock_data_1
    jmp     .rock_pointer_ready
.team_1_first
    lda     #rock_data_1 + rock_data_size
    jmp     .rock_pointer_ready
.advance_rock
    ldx     current_rock_number     ; See if the free-guard zone needs to be checked.
    cpx     #4
    bmi     .do_free_guard_check
    jmp     .no_fgz_violation
.do_free_guard_check
    mva     #0,$600                 ; Use this as a hit indicator.
    txa
    and     #1                      ; Figure out which player threw.
    beq     .check_odd_rocks
    lda     #2                      ; Check rocks 2 and 0.
    mvy     <#fgz_rock_2,$fe        ; Store the low byte of the record.
    jmp     .enforce_free_guard
.check_odd_rocks
    lda     #3                      ; Check rocks 3 and 1.
    mvy     <#fgz_rock_3,$fe        ; Store the low byte of the record.
.enforce_free_guard
    mvy     #6,$ff                  ; Store the high byte of the record.
    sta     $fd                     ; Store the rock number to check in $fd.
.fgz_enforce_loop
    ldy     #rock_data_size + 1     ; Check the high progress byte.
    lda     ($fe),y
    bne     .active_fgz_rock        ; If it's 1, this rock was active.
    dey                             ; Look at the low progress byte.
    lda     ($fe),y
    bne     .fgz_rock_is_guard      ; Definitely a guard.
    jmp     .skip_fgz_rock          ; If it's 0, this rock is inactive.
.active_fgz_rock
    iny                             ; Get the vertical.
    lda     ($fe),y
    tax
    ldy     #rock_data_size         ; Get the low byte.
    lda     ($fe),y
    jsr     rock_is_guard
    beq     .fgz_rock_is_guard
    jmp     .skip_fgz_rock          ; Not a guard, don't care.
.fgz_rock_is_guard
    ldy     $fd                     ; Get the rock number back.
    lda     rock_progress_high,y    ; If both bytes of progress are 0,
    beq     .fgz_rock_high_ok       ; the rock is now inactive.
    jmp     .skip_fgz_rock          ; Rock is still on the sheet.
.fgz_rock_high_ok
    lda     rock_progress,y
    beq     .fgz_rock_low_ok        ; Rock was knocked off the sheet.
    jmp     .skip_fgz_rock
.fgz_rock_low_ok
    mva     #1,$600                 ; Store a hit so we can put some text up.
    lda     #$ff                    ; Terminator value for the list of
    pha                             ; rocks to erase.
    ldx     #0                      ; Rock counter.
    mwa     #fgz_rock_0,$fe         ; Base address of the old rock data.
    mwa     #rock_data_1,$fc        ; Target address of the active rocks.
    mva     #rock_data_size,$fb     ; Bytes to copy.
.fgz_application_loop
    ldy     #rock_address + 1       ; Check the high byte of the address.
    lda     ($fc),y
    beq     .fgz_rock_dead          ; On 0, this rock is not active.
    pha                             ; Store the address for later erasure.
    dey
    lda     ($fc),y
    pha                             ; Save the low byte.
    ldy     #rock_offset
    lda     ($fc),y
    pha                             ; Finally, the offset.
.fgz_rock_dead
    jsr     copy_memory             ; Restore the old rock data.
    ldy     #rock_data_size         ; Restore the position data.
    lda     ($fe),y
    sta     rock_progress,x
    iny
    lda     ($fe),y
    sta     rock_progress_high,x
    iny
    lda     ($fe),y
    sta     rock_vertical,x
    inx
    cpx     #4
    beq     .fgz_application_done
    clc                             ; Update the addresses.
    lda     $fe
    adc     #rock_data_size + 3
    sta     $fe
    lda     $fc
    adc     #rock_data_size
    sta     $fc
    jmp     .fgz_application_loop
.fgz_application_done
    pla                             ; Start erasing rocks.
    bmi     .fgz_rock_erase_done    ; $ff is the terminator.
    tay                             ; Pixel offset.
    pla
    sta     $fe                     ; Low byte.
    pla
    sta     $ff                     ; High byte.
    jsr     erase_rock
    jmp     .fgz_application_done
.fgz_rock_erase_done
    mva     #0,$fd                  ; Now, redraw the first four rocks.
    mva     #rock_data_1,$fc        ; Save this address, draw_rock will
    pha                             ; clobber everything.
.fgz_rock_draw_loop
    ldy     #rock_address + 1       ; Check the high byte to see if the
    lda     ($fc),y                 ; rock is active.
    beq     .skip_fgz_rock_draw
    sta     $ff                     ; Set up the rock address.
    dey
    lda     ($fc),y
    sta     $fe
    ldy     #rock_flags             ; Set up the team.
    lda     ($fc),y
    and     #rock_team
    tax
    ldy     #rock_handle            ; Now the handle.
    lda     ($fc),y
    pha                             ; Save it.
    ldy     #rock_offset            ; Finally, the offset.
    lda     ($fc),y
    tay
    pla                             ; Get the handle back.
    jsr     draw_rock
.skip_fgz_rock_draw
    pla                             ; Retrive the data block.
    cmp     #rock_data_1 + #rock_data_size * 3
    beq     .fgz_rock_draw_done     ; All done after rock 4.
    clc
    adc     #rock_data_size         ; Advance to the next rock.
    pha                             ; Save it on the stack.
    sta     $fc                     ; Set up the address.
    mva     #0,$fd
    jmp     .fgz_rock_draw_loop
.fgz_rock_draw_done
    ldx     current_rock_number
    jmp     .no_free_guard_check
.skip_fgz_rock
    dec     $fd                     ; Drop two rock indices.
    dec     $fd
    bmi     .fgz_enforce_done       ; On negative, all done checking.
    lda     $fe                     ; Update the scrutinized rock address.
    sec
    sbc     [#rock_data_size + 3] * 2
    sta     $fe
    jmp     .fgz_enforce_loop
.fgz_enforce_done
    ldx     #3                      ; Update the first 4 rocks.
    mwa     [rock_data_1 + rock_data_size * 3],$fe
    mwa     fgz_rock_3,$fc          ; Set up the rock addresses.
    mva     #rock_data_size,$fb     ; Bytes to copy.
.fgz_record_loop
    jsr     copy_memory             ; Copy the rock record.
    ldy     #rock_data_size         ; Now get the horizontal and vertical.
    lda     rock_progress,x
    sta     ($fc),y
    lda     rock_progress_high,x
    iny
    sta     ($fc),y
    lda     rock_vertical,x
    iny
    sta     ($fc),y
    dex
    bmi     .fgz_record_loop_done
    sec
    lda     $fe                     ; Update the addresses.
    sbc     #rock_data_size
    sta     $fe
    lda     $fc
    sbc     #rock_data_size + 3
    sta     $fc
    jmp     .fgz_record_loop
.fgz_record_loop_done
    ldx     current_rock_number
.no_free_guard_check
    lda     $600                    ; If there was a violation, put up
    beq     .no_fgz_violation       ; some explanatory text.
    mwa     free_guard_text,$fc
    mwa     $1000,$fe
    jsr     copy_text_full
    ldy     #240
    jsr     wait_for_y_frames
.no_fgz_violation
    txa                             ; Need to erase the previous status rock.
    and     #1                      ; Even rocks are player 0.
    beq     .p0_erase
    mwa     p1_status_base,$fe      ; Set the status base address.
    jmp     .find_status_rock
.p0_erase
    mwa     p0_status_base,$fe      ; Set the status base address.
.find_status_rock
    txa                             ; Divide the rock number by 2
    lsr                             ; to get the individual status rock.
    tax
.status_loop
    cpx     #0                      ; Find the specific rock.
    beq     .erase_rock
    clc
    lda     $fe                     ; Advance two bytes.
    adc     #2
    bcc     .no_carry
    inc     $ff
.no_carry
    sta     $fe
    dex
    jmp     .status_loop
.erase_rock
    jsr     erase_status_rock
    lda     hammer_owner            ; Figure out which rock is next.
    bne     .rock_forward           ; Team 1 hammer always goes forward.
    lda     current_rock_number     ; If team 0 just shot, advance three rocks.
    and     #1
    beq     .rock_forward_x3
    lda     current_rock            ; Otherwise, go back one rock.
    sec
    sbc     #rock_data_size
    jmp     .rock_pointer_ready
.rock_forward_x3
    lda     current_rock
    clc
    adc     #rock_data_size * 3
    jmp     .rock_pointer_ready
.rock_forward
    lda     current_rock
    clc                             ; Point to the next rock.
    adc     #rock_data_size
.rock_pointer_ready
    cmp     #rock_overflow          ; See if this was the last rock.
    bcc     .continue_end
    jmp     finish_end
.continue_end
    sta     current_rock
    sta     active_rock_list
    ldy     #0                      ; Build the rest of the active rock list.
    ldx     #rock_data_1
.active_rock_loop
    lda     rock_address,x          ; See if this rock is active.
    bne     .active_rock
    lda     rock_address + 1,x
    beq     .next_rock
.active_rock
    txa
    sta     active_rock_list + 1,y  ; Save this rock for collision checks.
    iny
.next_rock
    txa
    clc
    adc     #rock_data_size
    tax
    cpx     #rock_overflow
    bne     .active_rock_loop
    lda     #0
    sta     active_rock_list + 1,y  ; Terminate the active rock list.
    lda     current_rock
    jsr     get_handedness
    bne     .left_handed
    ldy     current_rock
    lda     <#throw_address_right
    sta     rock_address,y
    lda     >#throw_address_right
    sta     rock_address + 1,y
    mwa     $294c,aim_cursor        ; Initialize the shot mode.
    mwa     $296f,aim_cursor_far
    mva     #96,last_stick          ; Approximate center.
    jmp     .handedness_done
.left_handed
    ldy     current_rock
    lda     <#throw_address_left
    sta     rock_address,y
    lda     >#throw_address_left
    sta     rock_address + 1,y
    mwa     $2d83,aim_cursor        ; Initialize the shot mode.
    mwa     $2da6,aim_cursor_far
    mva     #109,last_stick         ; Approximate center.
.handedness_done
    lda     #0
    sta     rock_offset,y
    lda     #6
    sta     rock_handle,y
    jsr     get_rock_number
    txa
    and     #1                      ; Even rocks are player 0.
    beq     .set_p0
    lda     #1
    sta     rock_flags,y
    jmp     .done_flags
.set_p0
    lda     #0
    sta     rock_flags,y
.done_flags
    jsr     get_rock_number
    stx     current_rock_number
    txa
    jsr     position_next_shot
    mva     #blink_length,blink_timer
    jsr     clear_active_rocks
    mva     #130,m0_low_h           ; Position the hogline.
    mva     #255,m3_low_h           ; Put the back line offscreen.
    mva     #shot_mode_aim,shot_mode
    mva     #0,trigger_timer        ; Reset the debounce timer.
    sta     rock_curl               ; Reset the curl tracking.
    sta     rock_curl + 1
    sta     valid_rock_struck       ; Clear the strike flag.
    sta     pebble_frames           ; Clear the pebble frames.
    sta     sweep_value             ; Clear the sweep.
    mva     #shot_end_delay_time,shot_end_delay
    mva     #23,pebble_friction     ; Reset the pebble.
    mva     #sweep_delay,sweep_timer
    mva     #0,sizep2_low           ; Normal player 2 size.
    sta     prior_low               ; Adjust priority.
    jsr     draw_curler
    jsr     reset_scroll
    jsr     create_sweeping_gauge
    jsr     ready_inputs
    mva     #$ff,last_key_pressed   ; Reset the keypad.
    jsr     draw_aim_scales
    rts

; Examine the current rock and game mode to
; prepare the input vectors and offsets.
ready_inputs subroutine
    lda     game_type
    cmp     #1
    beq     .mode_111               ; 1 vs. 1, 1 stick.
    cmp     #2
    beq     .mode_112               ; 1 vs. 1, 2 sticks.
    cmp     #3
    beq     .mode_222               ; 2 vs. 2, 2 sticks.
    cmp     #4
    beq     .mode_224               ; 2 vs. 2, 4 sticks.
    cmp     #5
    bne     .cpu_game
    jmp     .mode_44                ; 4 vs. 4
.cpu_game
    cmp     #9
    bne     .vs_cpu                 ; Player against CPU.
    jmp     .set_cpu_vectors        ; CPU vs CPU, easy!
.vs_cpu
    lda     current_rock_number     ; Even numbered rocks are the human player(s).
    and     #1
    beq     .human_player
    jmp     .set_cpu_vectors
.human_player
    lda     game_type
    cmp     #6                      ; We can reuse the player/player code here.
    beq     .mode_111
    cmp     #7
    beq     .mode_222
    jmp     .mode_44
.mode_111
    jsr     .stick_0_shoot          ; First stick for everything.
    jmp     .stick_0_sweep
.mode_112
    lda     current_rock_number     ; Even numbered rocks are stick 0,
    and     #1                      ; odd numbered rocks are stick 1.
    beq     .mode_111               ; Reuse this code.
    jsr     .stick_1_shoot
    jmp     .stick_1_sweep
.mode_222
    lda     current_rock_number     ; On rocks 0-3 and 8-11, stick 0
    cmp     #4                      ; shoots and 1 sweeps, opposite
    bmi     .shoot_0_sweep_1        ; for the others.
    cmp     #8
    bmi     .shoot_1_sweep_0
    cmp     #12
    bmi     .shoot_0_sweep_1
    jmp     .shoot_1_sweep_0
.mode_224
    lda     current_rock_number     ; Rocks 0, 2, 8, 10 are stick 0 shoot,
    beq     .shoot_0_sweep_1        ; stick 1 sweep.
    cmp     #2
    beq     .shoot_0_sweep_1
    cmp     #8
    beq     .shoot_0_sweep_1
    cmp     #10
    beq     .shoot_0_sweep_1
    cmp     #4                      ; Rocks 4, 6, 12, 14 are stick 1 shoot,
    beq     .shoot_1_sweep_0        ; stick 0 sweep.
    cmp     #6
    beq     .shoot_1_sweep_0
    cmp     #12
    beq     .shoot_1_sweep_0
    cmp     #14
    beq     .shoot_1_sweep_0
    cmp     #1                      ; Rocks 1, 3, 9, 11 are stick 2 shoot,
    beq     .shoot_2_sweep_3        ; stick 3 sweep.
    cmp     #3
    beq     .shoot_2_sweep_3
    cmp     #9
    beq     .shoot_2_sweep_3
    cmp     #11
    beq     .shoot_2_sweep_3
    jsr     .stick_3_shoot          ; All others go here.
    jmp     .stick_2_sweep
.shoot_2_sweep_3
    jsr     .stick_2_shoot
    jmp     .stick_3_sweep
.mode_44
    lda     current_rock_number
    cmp     #4                      ; On rocks 0-3, 0 shoots, 1 and 2 sweep.
    bmi     .shoot_0_sweep_12
    cmp     #8                      ; On rocks 4-7, 1 shoots, 0 and 2 sweep.
    bmi     .shoot_1_sweep_02
    cmp     #12                     ; On rocks 8-11, 2 shoots, 0 and 1 sweep.
    bmi     .shoot_2_sweep_01
    jsr     .stick_3_shoot          ; Otherwise, 3 shoots, 0 and 1 sweep.
    jmp     .stick_01_sweep
.shoot_0_sweep_12
    jsr     .stick_0_shoot
    jmp     .stick_12_sweep
.shoot_1_sweep_02
    jsr     .stick_1_shoot
    jmp     .stick_02_sweep
.shoot_2_sweep_01
    jsr     .stick_2_shoot
    jmp     .stick_01_sweep
.shoot_1_sweep_0
    jsr     .stick_1_shoot
    jmp     .stick_0_sweep
.shoot_0_sweep_1
    jsr     .stick_0_shoot
    jmp     .stick_1_sweep
.stick_0_shoot
    mva     TRIG0,trigger_buffer    ; Initialize the trigger buffer.
    lda     #p0vpot - #POT0
    sta     shooter_vpot
    lda     #p0hpot - #POT0
    sta     shooter_hpot
    mva     #0,shooter_trig
    mva     #$4,CONSOL
    mva     #1,$f0
    rts
.stick_1_shoot
    mva     TRIG1,trigger_buffer    ; Initialize the trigger buffer.
    lda     #p1vpot - #POT0
    sta     shooter_vpot
    lda     #p1hpot - #POT0
    sta     shooter_hpot
    mva     #1,shooter_trig
    mva     #$5,CONSOL
    mva     #2,$f0
    rts
.stick_2_shoot
    mva     TRIG2,trigger_buffer    ; Initialize the trigger buffer.
    lda     #p2vpot - #POT0
    sta     shooter_vpot
    lda     #p2hpot - #POT0
    sta     shooter_hpot
    mva     #2,shooter_trig
    mva     #$6,CONSOL
    mva     #3,$f0
    rts
.stick_3_shoot
    mva     TRIG3,trigger_buffer    ; Initialize the trigger buffer.
    lda     #p3vpot - #POT0
    sta     shooter_vpot
    lda     #p3hpot - #POT0
    sta     shooter_hpot
    mva     #3,shooter_trig
    mva     #$7,CONSOL
    mva     #4,$f0
    rts
.stick_0_sweep
    mwa     one_stick_sweep,sweep_value_vector
    lda     #p0vpot - #POT0
    sta     sweeper_vpot_1
    mva     #1,$f1
    mva     #0,$f2
    mwa     shot_text_one_sweeper,$fc
    jmp     .set_player_vectors
.stick_1_sweep
    mwa     one_stick_sweep,sweep_value_vector
    lda     #p1vpot - #POT0
    sta     sweeper_vpot_1
    mva     #2,$f1
    mva     #0,$f2
    mwa     shot_text_one_sweeper,$fc
    jmp     .set_player_vectors
.stick_2_sweep
    mwa     one_stick_sweep,sweep_value_vector
    lda     #p2vpot - #POT0
    sta     sweeper_vpot_1
    mva     #3,$f1
    mva     #0,$f2
    mwa     shot_text_one_sweeper,$fc
    jmp     .set_player_vectors
.stick_3_sweep
    mwa     one_stick_sweep,sweep_value_vector
    lda     #p3vpot - #POT0
    sta     sweeper_vpot_1
    mva     #4,$f1
    mva     #0,$f2
    mwa     shot_text_one_sweeper,$fc
    jmp     .set_player_vectors
.stick_01_sweep
    mwa     two_stick_sweep,sweep_value_vector
    lda     #p0vpot - #POT0
    sta     sweeper_vpot_1
    lda     #p1vpot - #POT0
    sta     sweeper_vpot_2
    mva     #1,$f1
    mva     #2,$f2
    mwa     shot_text_two_sweepers,$fc
    jmp     .set_player_vectors
.stick_02_sweep
    mwa     two_stick_sweep,sweep_value_vector
    lda     #p0vpot - #POT0
    sta     sweeper_vpot_1
    lda     #p2vpot - #POT0
    sta     sweeper_vpot_2
    mva     #1,$f1
    mva     #3,$f2
    mwa     shot_text_two_sweepers,$fc
    jmp     .set_player_vectors
.stick_12_sweep
    mwa     two_stick_sweep,sweep_value_vector
    lda     #p1vpot - #POT0
    sta     sweeper_vpot_1
    lda     #p2vpot - #POT0
    sta     sweeper_vpot_2
    mva     #2,$f1
    mva     #3,$f2
    mwa     shot_text_two_sweepers,$fc
.set_player_vectors
    mwa     $1000,$fe
    jsr     copy_text_full
    lda     $f0
    clc
    adc     #$10
    sta     $1007
    lda     $f2
    beq     .one_sweeper
    adc     #$10
    sta     $101e
    lda     $f1
    adc     #$10
    sta     $101c
    jmp     .finish_vectors
.one_sweeper
    lda     $f1
    adc     #$10
    sta     $101e
.finish_vectors
    mwa     get_player_aim,aim_value_vector
    mwa     get_player_trig,aim_trig_vector
    mwa     get_player_weight,weight_value_vector
    mwa     get_player_curl,curl_value_vector
    mwa     get_player_keypress,key_value_vector
    rts
.set_cpu_vectors
    mwa     cpu_shot_text,$fc
    mwa     $1000,$fe
    mva     #$04,CONSOL             ; Allow stick 0 to pause the game.
    jsr     copy_text_full
    jsr     reset_cpu_player
    mwa     get_cpu_aim,aim_value_vector
    mwa     get_cpu_trig,aim_trig_vector
    mwa     get_cpu_weight,weight_value_vector
    mwa     get_cpu_curl,curl_value_vector
    mwa     get_cpu_keypress,key_value_vector
    mwa     cpu_sweep,sweep_value_vector
    rts

; Place the weight mode text on the screen.  Uses the mode in
; the accumulator.
place_weight_mode_text subroutine
    cmp     #weight_mode_guard
    beq     .weight_guard
    cmp     #weight_mode_draw
    beq     .weight_draw
    mwa     takeout_mode_sound,$fe
    jsr     play_sound_1
    mwa     weight_takeout_text,$fe
    jmp     .draw_text
.weight_guard
    mwa     guard_mode_sound,$fe
    jsr     play_sound_1
    mwa     weight_guard_text,$fe
    jmp     .draw_text
.weight_draw
    mwa     draw_mode_sound,$fe
    jsr     play_sound_1
    mwa     weight_draw_text,$fe
.draw_text
    mwa     weight_text,$fc
    mva     #8,$fb
    ldx     #5
.loop
    jsr     copy_memory
    dex
    beq     .done
    lda     $fc
    clc
    adc     #bytes_per_row
    sta     $fc
    lda     $fd
    adc     #0
    sta     $fd
    lda     $fe
    clc
    adc     #8
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    jmp     .loop
.done
    rts

; Remove the weight gauge from the playfield.
erase_weight_gauge subroutine
    ldx     #5                      ; 5 rows in the text.
    mwa     weight_text,$fe
.erase_text_loop_outer
    ldy     #7                      ; 8 bytes per row.
    lda     #0                      ; Write 0s.
.erase_text_loop_inner
    sta     ($fe),y
    dey
    bpl     .erase_text_loop_inner
    dex
    beq     .erase_text_done
    lda     $fe                     ; Update the row address.
    clc
    adc     #bytes_per_row
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    jmp     .erase_text_loop_outer
.erase_text_done
    ldx     #6                      ; 6 rows.
    mwa     weight_meter_left,$fe
.scale_loop_outer
    ldy     #0
    tya                             ; Write 0s.
.scale_loop_inner
    sta     ($fe),y
    iny
    cpy     #weight_meter_right - #weight_meter_left + 1
    bne     .scale_loop_inner
    dex
    beq     .done
    lda     $fe                     ; Update the row address.
    clc
    adc     #bytes_per_row
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    jmp     .scale_loop_outer
.done
    rts

; Add the rock in x to the end of the active rocks list.
activate_rock subroutine
    ldy     #0
.loop
    txa
    cmp     rock_list,y             ; See if the rock is already there.
    beq     .already_in_list
    lda     rock_list,y             ; Find the end of the list.
    beq     .found_end
    iny
    jmp     .loop
.found_end
    stx     rock_list,y             ; Put the rock in the list.
.already_in_list
    rts

; Remove the rock in x from the active rocks list.  If x
; isn't is the list, this subroutine will do bad things.
deactivate_rock subroutine
    ldy     #0
    txa
.loop
    cmp     rock_list,y             ; Find the rock in the list.
    beq     .found_rock
    iny
    jmp     .loop
.found_rock
    cpy     #15                     ; If this is the end of the list,
    beq     .end_list               ; cap it with a 0.
    iny                             ; Back the values up by one slot.
    lda     rock_list,y
    sta     rock_list - 1,y
    jmp     .found_rock
.end_list
    lda     #0
    sta     rock_list,y
    rts

; Clear the active rock list.
clear_active_rocks subroutine
    ldy     #15
    lda     #0
.loop
    sta     rock_list,y
    dey
    bpl     .loop
    rts

; Prepare the curler graphics.
draw_curler subroutine
    ldx     #0
    ldy     #$8b
.erase_loop
    lda     #0
    sta     $1400,y
    sta     $1500,y
    sta     $1600,y
    sta     $1700,y
    inx
    iny
    cpx     #64
    bne     .erase_loop
    jsr     get_handedness
    bne     .left_handed
    ldx     #$a2                    ; Right handed base address.
    jmp     .draw_common
.left_handed
    ldx     #$af                    ; Left handed base address.
.draw_common
    stx     curler_p2_address
    ldy     current_rock_number
    lda     rock_shooter_map,y
    tay
    jsr     draw_curler_common
    mva     #$36,p1_low_h
    mva     #$33,p2_low_h
    mva     #$31,p3_low_h
    jsr     get_rock_number
    ldy     rock_shooter_map,x
    lda     curler_hair_color,y
    sta     pm1_low_color_1
    sta     pm1_low_color_2
    sta     pm1_low_color_3
    lda     curler_jacket_color,y
    sta     pm2_low_color_1
    sta     pm2_low_color_2
    sta     pm2_low_color_3
    lda     curler_pants_color,y
    sta     pm3_low_color
    rts

; Move the curler one pixel down.
move_curler_down subroutine
    inc     curler_p2_address
    ldx     #0
    ldy     #$c1
.loop
    lda     $1400,y
    sta     $1401,y
    lda     $1500,y
    sta     $1501,y
    lda     $1600,y
    sta     $1601,y
    lda     $1700,y
    sta     $1701,y
    dey
    inx
    cpx     #50
    bne     .loop
    lda     #0
    sta     $1400,y
    sta     $1500,y
    sta     $1600,y
    sta     $1700,y
    rts

; Move the curler one pixel up.
move_curler_up subroutine
    dec     curler_p2_address
    ldx     #0
    ldy     #$8b
.loop
    lda     $1401,y
    sta     $1400,y
    lda     $1501,y
    sta     $1500,y
    lda     $1601,y
    sta     $1600,y
    lda     $1701,y
    sta     $1700,y
    iny
    inx
    cpx     #55
    bne     .loop
    lda     #0
    sta     $1400,y
    sta     $1500,y
    sta     $1600,y
    sta     $1700,y
    rts

; Process the conclusion of an end.
finish_end subroutine
    ldx     #$ff                    ; Reset the stack, there are some
    txs                             ; jsrs that need to be terminated.
    mva     #0,AUDCTL               ; Play the crowd noise.
    mwa     crowd_sound,$fe
    jsr     play_sound_1
    ldy     #47                     ; Set the scoring computation area
    txa                             ; to all $ff.  This marks rocks as
.clear_loop                         ; not considered.
    sta     rock_scoring,y
    dey
    bpl     .clear_loop
.rock_loop
    iny                             ; Start from rock 0.
    lda     rock_progress_high,y    ; If the high byte of the progress
    bne     .evaluate_rock          ; is 0, the rock is not in the house.
    jmp     .next_rock
.evaluate_rock
    jsr     rock_in_house
    beq     .rock_in_house
    jmp     .next_rock              ; Not in the house, skip it.
.rock_in_house
    jsr     distance_from_button
    mva     $fe,$fc                 ; Place the result for comparison.
    mva     $ff,$fd
    ldx     #0                      ; Offset into the scoring list.
.compare_loop
    lda     rock_scoring,x          ; Measure against the next rock.
    sta     $fa
    lda     rock_scoring + 1,x
    sta     $fb
    jsr     compare_16
    beq     .insert_rock            ; This rock is at least as close as the
    bpl     .insert_rock            ; one in the list.
    inx                             ; This rock is behind the one in the list.
    inx
    inx                             ; Move three bytes forward in the list.
    jmp     .compare_loop
.insert_rock
    stx     $f1                     ; Save the insertion point.
.find_end_loop
    lda     rock_scoring + 2,x      ; Find the end of the list.
    bmi     .found_end              ; Empty spots have $ff as the rock number.
    inx
    inx
    inx                             ; Move up to the next position.
    jmp     .find_end_loop
.found_end
    cpx     $f1                     ; See if we've reached the insertion point.
    beq     .done_list_shift
    lda     rock_scoring - 3,x      ; Move the previous rock forward.
    sta     rock_scoring,x
    lda     rock_scoring - 2,x
    sta     rock_scoring + 1,x
    lda     rock_scoring - 1,x
    sta     rock_scoring + 2,x
    dex
    dex
    dex                             ; Move back a position.
    jmp     .found_end
.done_list_shift
    lda     $fe                     ; Store the distance.
    sta     rock_scoring,x
    lda     $ff
    sta     rock_scoring + 1,x
    lda     $f0                     ; Store the rock number.
    sta     rock_scoring + 2,x
    tay                             ; Get the rock number back in y.
.next_rock
    cpy     #15
    beq     .score_rocks
    jmp     .rock_loop
.score_rocks
    lda     #$ff                    ; ff terminates the rock scoring list.
    pha                             ; Build the score list on the stack.
    ldx     #0                      ; Start at the beginning of the rock list.
    lda     rock_scoring + 2,x      ; Get the number of the closest rock.
    bmi     .end_scoring            ; If it's ff, this is a blank end.
    pha                             ; Push the first rock_number.
    and     #1                      ; Odd or even determines the team.
    sta     $f0                     ; Store the scoring team.
.score_loop
    inx
    inx
    inx                             ; Go to the next list entry.
    lda     rock_scoring + 2,x      ; Get the rock number.
    bmi     .end_scoring            ; Number ff is the list end.
    and     #1                      ; Reduce to odd/even.
    cmp     $f0                     ; Test against the scoring team.
    bne     .end_scoring            ; If it doesn't match, the score is complete.
    lda     rock_scoring + 2,x      ; Save the rock number.
    pha
    jmp     .score_loop
.end_scoring
    ldy     #0                      ; Calculate the number of points to award.
.score_tally_loop
    pla                             ; Get a rock off the stack.
    sta     rock_scoring,y          ; Write it to a list so it can be blinked.
    bmi     .done_score_tally       ; ff terminates the list.
    iny                             ; Award a point.
    jmp     .score_tally_loop
.done_score_tally
    sty     $f0                     ; Save the score.
    ldy     current_end             ; Check the end number.
    cpy     #1                      ; On the first end, there is no score
    beq     .start_new_score        ; to add to.
    cpy     game_length             ; On an extra end, add to the current score.
    bmi     .carry_last_score
    beq     .carry_last_score
    dey
    jmp     .keep_old_score
.carry_last_score
    dey                             ; Retrieve the score from the
    jsr     get_score_offset        ; previous end.
    lda     score_base_0,x          ; Copy the old score into the new end.
    sta     score_base_0 + 4,x
    lda     score_base_0 - 1,x
    sta     score_base_0 + 3,x
    lda     score_base_1,x
    sta     score_base_1 + 4,x
    lda     score_base_1 - 1,x
    sta     score_base_1 + 3,x
    jmp     .add_new_score
.start_new_score
    lda     #$10                    ; Character 0.
    sta     score_base_0 + 2
    sta     score_base_1 + 2
.add_new_score
    ldy     current_end
.keep_old_score
    jsr     get_score_offset
    txa
    tay
    lda     rock_scoring            ; Test the team.
    and     #1
    beq     .score_p0
    mwa     score_base_1,$fe        ; Store the score base.
    jmp     .finalize_score
.score_p0
    mwa     score_base_0,$fe        ; Store the score base.
.finalize_score
    lda     ($fe),y
    clc
    adc     $f0                     ; Add the score.
    cmp     #$1a                    ; See if the 10s digit needs to be updated.
    bpl     .update_10s
    sta     ($fe),y
    jmp     .scoring_done
.update_10s
    sec
    sbc     #10
    sta     ($fe),y
    dey
    lda     ($fe),y
    beq     .blank_digit
    adc     #0                      ; Carry bit is set, increments.
    jmp     .scoring_done
.blank_digit
    lda     #$11                    ; Character 1.
.scoring_done
    sta     ($fe),y
    mvy     #4,$f2                  ; Blink 4 times.
.blink_rock_loop
    ldy     #0                      ; Iterate the rock numbers.
.blank_rock_loop
    ldx     rock_scoring,y          ; Get the next rock number.
    bmi     .blank_rock_done        ; List ends at $ff.
    jsr     .prep_rock_location
    jsr     erase_rock              ; Blank the rock.
    ldy     $f1                     ; Get the rock number back.
    iny                             ; Go to the next rock.
    jmp     .blank_rock_loop
.blank_rock_done
    ldy     #30                     ; Pause for half a second.
    jsr     wait_for_y_frames
    ldy     #0                      ; Iterate the rock numbers.
.draw_rock_loop
    ldx     rock_scoring,y          ; Get the next rock number.
    bmi     .draw_rock_done         ; List ends at $ff.
    jsr     .prep_rock_location
    lda     rock_handle,x
    pha
    lda     rock_flags,x
    and     #rock_team
    tax
    pla
    jsr     draw_rock               ; Redraw the rock.
    ldy     $f1                     ; Get the rock number back.
    iny                             ; Go to the next rock.
    jmp     .draw_rock_loop
.draw_rock_done
    ldy     #30                     ; Pause for half a second.
    jsr     wait_for_y_frames
    dec     $f2                     ; Test the blink count.
    beq     .blink_rock_done
    jmp     .blink_rock_loop
.blink_rock_done
    lda     rock_scoring            ; Check for a change in the hammer.
    bmi     .hammer_change_done     ; Blank end, no change.
    and     #1
    cmp     hammer_owner
    bne     .hammer_change_done     ; A steal!  No hammer change.
    eor     #1                      ; Flip the hammer.
    sta     hammer_owner
.hammer_change_done
    ldy     current_end             ; Check for end of game.
    cpy     game_length
    bpl     .check_for_tie
    iny
    sty     current_end             ; Update the end.
    jmp     start_new_end
.check_for_tie
    mwa     score_base_0,$fe        ; Set up the score locations.
    mwa     score_base_1,$fc
    ldy     game_length             ; Get the offset to the last end.
    jsr     get_score_offset
    txa
    tay
    lda     ($fe),y                 ; Check the 1s digit.
    cmp     ($fc),y
    bne     .game_over              ; No tie, game is over.
    dey                             ; Check the 10s digit.
    lda     ($fe),y
    cmp     ($fc),y
    bne     .game_over              ; No tie, game is over.
    ldy     current_end             ; Go to end +1 if not already there.
    cpy     #11
    beq     .already_extra_end
    inc     current_end
    lda     #$38
    sta     score_base_label,x      ; Put an 'X' on the scoreboard.
.already_extra_end
    jmp     start_new_end
.game_over
    mwa     game_over_text,$fc
    mwa     $1000,$fe
    jsr     copy_text_full
    mva     #$4,CONSOL              ; Enable stick 1's keypad.
.game_over_loop
    jsr     wait_for_vb
    lda     last_key_pressed
    cmp     #$0e
    bne     .no_reset
    jmp     start_title
.no_reset
    mva     #$ff,last_key_pressed
    jmp     .game_over_loop
 
 ; Subroutine for blanking/drawing scoring rocks.
.prep_rock_location
    jsr     get_rock_pointer_from_x ; Get the rock address.
    lda     rock_address,x          ; Store the graphics address.
    sta     $fe
    lda     rock_address + 1,x
    sta     $ff
    sty     $f1                     ; Save the rock number.
    ldy     rock_offset,x
    rts
   
; Get a rock's distance from the button.  Expects
; the rock number in y, and that the rock has
; already passed a rock_in_house check.
; Places the distance in $fe-$ff.
distance_from_button subroutine
    sty     $f0                     ; Save y.
    lda     $f2                     ; Get the horizontal distance.
    asl                             ; Double the distance, pixels are wide.
    jsr     square                  ; Square the distance.
    sta     $fa                     ; Store the value.
    stx     $fb
    ldy     $f0                     ; Get the rock number back.
    lda     rock_vertical,y         ; Now get the vertical distance.
    cmp     #button_top             ; Figure out which side of the button
    bcc     .top_side               ; this rock is on.
    beq     .top_side
    sec
    sbc     #button_bottom          ; Get the vertical distance.
    jmp     .store_vertical
.top_side
    lda     #button_top
    sec
    sbc     rock_vertical,y         ; Get the vertical distance.
.store_vertical
    jsr     square                  ; Square the distance.
    sta     $fc                     ; Save the value.
    stx     $fd
    jsr     add_16                  ; Add the two distances.
    ldy     $f0                     ; Restore y.
    rts

; Calculate the scoreboard offset for the end in y.
; The result is placed in x.
get_score_offset subroutine
    lda     #2                      ; Start 2 bytes in.
    clc
.loop
    dey
    beq     .done
    adc     #4                      ; 4 bytes between ends.
    jmp     .loop
.done
    tax
    rts
   
; Determine the handedness of the current shooter.
; Places a 1 in the accumulator for a lefty, 0
; for a right-handed shooter.
get_handedness subroutine
    jsr     get_rock_number
    lda     rock_shooter_map,x
    tay
    lda     curler_handedness,y
    rts

; Deferred vblank handler.
game_vblank subroutine
    jsr     process_sounds          ; Handle the sounds.
    mva     #0,ATRACT               ; Kill the attract timer.
    ldy     key_debounce            ; Process the debounce timer.
    beq     .no_debounce
    dec     key_debounce
.no_debounce
    sta     VDSLST                  ; Reset the hblank interrupt.
    mva     #$a0,VDSLST + 1
    mva     #ring_base,HPOSP0       ; Position the players for the top rings.
    mva     #ring_base + 8,HPOSP1
    mva     #ring_base + 16,HPOSP2
    mva     #ring_base + 24,HPOSP3
    mva     #front_hog_line,HPOSM0  ; Position missile 0 for the top hog line.
    mva     #end_cap_line,HPOSM1    ; Position missile 1 for the score capper.
    mva     #back_line,HPOSM3       ; Position missile 3 for the top back line.
    mva     #4,PRIOR                ; Playfield priority over all.
    mva     #0,SIZEP2               ; Reset the player 2 size.
    jmp     ENDINT

; 132 feet
; 330 pixels
; 83 bytes per row
; 5810 bytes total

; $1000 top text
; $1028 score text top (end numbers)
; $1050 first score line
; $1078 team 1 scores
; $10a0 second score line
; $10c8 team 2 scores
; $10f0 third score line
; $3700 bottom display

; 70 pixels from shot start to hogline


; Use speeds 68+ for shooting.

; Guard speed range: 68 - 76
; Draw speed range: 75 - 80
; Takeout speed range: 80 - 100

