; This file is part of RealSports Curling.

; RealSports Curling is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; RealSports Curling is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with RealSports Curling.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2018 Average Software.

; Various interrupt handlers

; First hblank interrupt on the game screen.  Changes the
; colors to display the upper ice view.
    org     $a000
    pha
    txa
    pha

    lda     #ice_color
    ldx     #rock_color_1
    sta     WSYNC
    sta     COLBK
    stx     COLPF1
    mva     #$19,VDSLST             ; Set the next interrupt.

    pla
    tax
    pla
    rti

; Second interrupt on the game screen.  Changes the
; player 1 and 2 colors to red for the inner ring.
    org     $a019
    pha

    mva     #$2c,VDSLST             ; Set the next interrupt.
    lda     #inner_ring_color
    sta     WSYNC
    sta     COLPM1
    sta     COLPM2

    pla
    rti

; Third interrupt on the game screen.  Changes the
; background color to black for the center line.
    org     $a02c
    pha

    mva     #$3c,VDSLST             ; Set the next interrupt.
    lda     #0                      ; Change to black.
    sta     WSYNC
    sta     COLBK

    pla
    rti

; Fourth interrupt on the game screen.  Changes the
; background color back to the ice color.
    org     $a03c
    pha

    mva     #$59,VDSLST             ; Set the next interrupt.
    lda     #ice_color
    sta     WSYNC
    sta     COLBK

    pla
    rti

; Fifth interrupt on the game screen.  Changes the
; player 1 and 2 colors back to blue for the outer ring.
    org     $a059
    pha

    mva     #$7c,VDSLST             ; Set the next interrupt.
    lda     #outer_ring_color
    sta     WSYNC
    sta     COLPM1
    sta     COLPM2

    pla
    rti

; Fourth interrupt on the game screen.  Changes back to
; the text colors for the scoreboard.  Also change
; player 1 to white for the end-cap missile, position
; players 1, 2, 3 for the various scoreboard elements.
    org     $a07c
    pha
    txa
    pha

    lda     #0                      ; Background to black.
    ldx     #$0f                    ; P1 and PF1 to white.
    sta     WSYNC
    sta     COLBK
    sta     PRIOR
    stx     COLPM1
    stx     COLPF1
    mva     #51,HPOSP2              ; Position the team flags.
    sta     HPOSP3
    mva     hammer_hpos,HPOSP1      ; Position the hammer.
    mva     #rock_color_0,COLPM2    ; Set the team colors.
    mva     #rock_color_1,COLPM3
    mva     #$b3,VDSLST             ; Set the next interrupt.

    pla
    tax
    pla
    rti

; Fifth interrupt on the game screen.  Position the players
; and missiles for the bottom sheet.
    org     $a0b3
    pha

    mva     #$ef,VDSLST             ; Set the next interrupt.
    mva     m3_low_h,HPOSM3         ; The order of these moves
    mva     m0_low_h,HPOSM0         ; is significant, timing is
    mva     p0_low_h,HPOSP0         ; problem.
    mva     p1_low_h,HPOSP1
    mva     p2_low_h,HPOSP2
    mva     p3_low_h,HPOSP3
    mva     m1_low_h,HPOSM1
    mva     m2_low_h,HPOSM2

    pla
    rti

; Sixth interrupt on the game screen.  Changes back to
; the ice colors for the bottom sheet.  Also sets
; the first of the player/missile colors.
    org     $a0ef
    pha
    txa
    pha

    lda     #ice_color
    ldx     #rock_color_1
    sta     WSYNC
    sta     COLBK
    stx     COLPF1
    mva     pm1_low_color_1,COLPM1
    mva     pm2_low_color_1,COLPM2
    mva     pm3_low_color,COLPM3
    mva     #$29,VDSLST             ; Set the next interrupt.
    inc     VDSLST + 1
    mva     sizep2_low,SIZEP2       ; Set the player 2 size.
    mva     prior_low,PRIOR         ; Set the lower priority.

    pla
    tax
    pla
    rti

; Seventh interrupt on the game screen.  Changes
; PM1 to its second color.
    org     $a129
    pha

    mva     #$36,VDSLST             ; Set the next interrupt.
    lda     pm1_low_color_2
    sta     COLPM1

    pla
    rti

; Eighth interrupt on the game screen.  Changes
; PM2 to its second color.
    org     $a136
    pha

    mva     #$4c,VDSLST             ; Set the next interrupt.
    lda     pm2_low_color_2
    sta     COLPM2

    pla
    rti

; Ninth interrupt on the game screen.  Changes the background
; color to black for the center line.
    org     $a14c
    pha

    lda     #0                      ; Change to black.
    sta     WSYNC
    sta     COLBK
    mva     #$5c,VDSLST             ; Set the next interrupt.

    pla
    rti

; Tenth interrupt on the game screen.  Changes the background
; color back to the ice color.
    org     $a15c
    pha

    lda     #ice_color
    sta     WSYNC
    sta     COLBK
    mva     #$6c,VDSLST             ; Set the next interrupt.

    pla
    rti

; Eleventh interrupt on the game screen.  Changes PM2
; to its third and final color.
    org     $a16c
    pha

    lda     pm2_low_color_3
    sta     COLPM2
    mva     #$79,VDSLST             ; Set the next interrupt.

    pla
    rti

; Twelfth interrupt on the game screen. Changes PM1
; to its third and final color.
    org     $a179
    pha

    lda     pm1_low_color_3
    sta     COLPM1
    mva     #$86,VDSLST             ; Set the next interrupt.

    pla
    rti

; Thirteenth interrupt on the game screen.  Moves P3 into position
; for the sweeping gauge.
    org     $a186
    pha
    
    lda     #50
    sta     HPOSP3
    lda     #outer_ring_color
    sta     COLPM3
    mva     #$98,VDSLST

    pla
    rti

; Fourteenth interrupt on the game screen.  Moves P0 into position
; for the sweeping gauge.
    org     $a198
    pha

    lda     #58
    sta     HPOSP0
    lda     #outer_ring_color
    sta     COLPM0
    mva     #$aa,VDSLST

    pla
    rti

; Fifteenth interrupt on the game screen.  Changes the background
; to black for the bottom rock display.
    org     $a1aa
    pha

    lda     #0
    sta     WSYNC
    sta     COLBK
    mva     #$ba,VDSLST             ; Set the next interrupt.

    pla
    rti

; Sixteenth interrupt on the game screen.  Change the colors
; of the status rocks to grey.
    org     $a1ba
    pha

    lda     #$0a
    sta     WSYNC
    sta     COLPF0
    sta     COLPF1
    mva     #$cd,VDSLST             ; Set the next interrupt.

    pla
    rti

; Last interrupt on the game screen.  Further shading
; of the status rocks.
    org     $a1cd
    pha

    lda     #$06
    sta     WSYNC
    sta     COLPF0
    sta     COLPF1

    pla
    rti

; Keypad interrupt.
    org     $af00
keypad_interrupt subroutine
    ldy     key_debounce
    bne     .wait_on_debounce
    cmp     #$ff
    beq     .store_key
    mvy     #30,key_debounce
.store_key
    sta     last_key_pressed
.wait_on_debounce
    jmp     ENDINT

