; This file is part of RealSports Curling.

; RealSports Curling is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; RealSports Curling is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with RealSports Curling.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2018 Average Software.

; Miscellaneous rock routines

; Draw a rock.  The byte with top-left corner of the rock
; should be at $fe.  The pixel offset should be in y,
; with the team identifier in x.  The handle position
; should be passed in the accumulator.
draw_rock subroutine
    sty     $fd                     ; Save the offset.
    sta     $fc                     ; Save the handle orientation.
    cpx     #1                      ; Get the rock color.
    beq     .team1_rock
    lda     #$54                    ; Start with three pixels of team 1 color.
    jmp     .create_top_row
.team1_rock
    lda     #$a8                    ; Start with three pixels of team 0 color.
.create_top_row
    tsx                             ; Save the current stack pointer.
    stx     $fa                     ; It's used if we don't need the second byte.
    sta     $fb                     ; Save the color pattern for later.
    ldx     $fc                     ; Check the handle orientation.
    bne     .no_0_handle            ; Position 0 is straight up.
    lda     #$30                    ; One black pixel.
    jmp     .shift_top
.no_0_handle
    and     #$30                    ; Trim down to one pixel for the top row.
.shift_top
    pha                             ; Save this row for the possible second byte.
.shift_top_loop
    dey                             ; Shift right until y goes negative.
    bmi     .done_shift_top
    lsr
    lsr
    jmp     .shift_top_loop
.done_shift_top
    iny                             ; Set y back to 0.
    ora     ($fe),y                 ; OR with the existing byte.
    sta     ($fe),y                 ; Update the playfield.
    lda     $fe                     ; Update the address.
    clc
    adc     #bytes_per_row
    bcc     .no_carry1
    inc     $ff
.no_carry1
    sta     $fe
    lda     $fb                     ; Get the color pattern back.
    cpx     #0                      ; Check the handle orientation.
    bne     .no_0_handle_2
    ora     #$30                    ; Position 0, blacken the center pixel.
    jmp     .shift_row_2
.no_0_handle_2
    cpx     #1
    bne     .no_1_handle
    ora     #$0c                    ; Position 1, blacken the right pixel.
    jmp     .shift_row_2
.no_1_handle
    cpx     #7
    bne     .shift_row_2
    ora     #$c0                    ; Position 7, blacken the left pixel.
.shift_row_2
    ldy     $fd                     ; Get the offset.
    pha                             ; Save this for byte 2.
.shift_row_2_loop
    dey                             ; Shift right until y goes negative.
    bmi     .done_shift_row_2
    lsr
    lsr
    jmp     .shift_row_2_loop
.done_shift_row_2
    iny                             ; Set y back to 0.
    ora     ($fe),y                 ; OR with the existing byte.
    sta     ($fe),y                 ; Update the playfield.
    lda     $fe                     ; Update the address.
    clc
    adc     #bytes_per_row
    bcc     .no_carry2
    inc     $ff
.no_carry2
    sta     $fe
    lda     $fb                     ; Get the color pattern back.
    cpx     #2                      ; Check the handle orientation.
    bne     .no_2_handle
    ora     #$3c                    ; Position 2, blacken the right and center pixels.
    jmp     .shift_row_3
.no_2_handle
    cpx     #6
    bne     .handle_center
    ora     #$f0                    ; Position 6, blacken the left and center pixels.
    jmp     .shift_row_3
.handle_center
    ora     #$30                    ; Center pixel is always black.
.shift_row_3
    ldy     $fd                     ; Get the offset.
    pha                             ; Save this for byte 2.
.shift_row_3_loop
    dey                             ; Shift right until y goes negative.
    bmi     .done_shift_row_3
    lsr
    lsr
    jmp     .shift_row_3_loop
.done_shift_row_3
    iny                             ; Set y back to 0.
    ora     ($fe),y                 ; OR with the existing byte.
    sta     ($fe),y                 ; Update the playfield.
    lda     $fe                     ; Update the address.
    clc
    adc     #bytes_per_row
    bcc     .no_carry3
    inc     $ff
.no_carry3
    sta     $fe
    lda     $fb                     ; Get the color pattern back.
    cpx     #3                      ; Check the handle orientation.
    bne     .no_3_handle
    ora     #$0c                    ; Position 3, blacken the right pixel.
    jmp     .shift_row_4
.no_3_handle
    cpx     #4
    bne     .no_4_handle
    ora     #$30                    ; Position 4, blacken the center pixel.
    jmp     .shift_row_4
.no_4_handle
    cpx     #5
    bne     .shift_row_4
    ora     #$c0                    ; Position 5, blacken the left pixel.
.shift_row_4
    ldy     $fd                     ; Get the offset.
    pha                             ; Save this for byte 2.
.shift_row_4_loop
    dey                             ; Shift right until y goes negative.
    bmi     .done_shift_row_4
    lsr
    lsr
    jmp     .shift_row_4_loop
.done_shift_row_4
    iny                             ; Set y back to 0.
    ora     ($fe),y                 ; OR with the existing byte.
    sta     ($fe),y                 ; Update the playfield.
    lda     $fe                     ; Update the address.
    clc
    adc     #bytes_per_row
    bcc     .no_carry4
    inc     $ff
.no_carry4
    sta     $fe
    lda     $fb                     ; Get the color pattern back.
    cpx     #4                      ; Check the handle orientation.
    bne     .no_4_handle2
    lda     #$30                    ; One black pixel for position 4.
    jmp     .shift_bottom
.no_4_handle2
    and     #$30                    ; Trim down to one pixel.
.shift_bottom
    ldy     $fd                     ; Get the offset.
    pha                             ; Save this for byte 2.
.shift_bottom_loop
    dey                             ; Shift right until y goes negative.
    bmi     .done_shift_bottom
    lsr
    lsr
    jmp     .shift_bottom_loop
.done_shift_bottom
    iny                             ; Set y back to 0.
    ora     ($fe),y                 ; OR with the existing byte.
    sta     ($fe),y                 ; Update the playfield.
    ldy     $fd                     ; Get the offset back.
    cpy     #2                      ; An offset less than 2 doesn't stride a byte.
    bpl     .do_second_byte
    ldx     $fa                     ; Restore the stack pointer to knock
    txs                             ; the five rows off the stack.
    rts                             ; All done.
.do_second_byte
    inc     $fe                     ; Move over one byte.
    bne     .no_overflow
    inc     $ff
.no_overflow
    lda     #4                      ; 4 - offset = bytes to shift.
    sec
    sbc     $fd
    sta     $fd
    ldx     #5                      ; Draw 5 bytes.
.byte_loop
    pla                             ; Get the byte to draw off the stack.
    ldy     $fd                     ; Get the offset.
.offset_loop
    dey                             ; Shift left until y goes negative.
    bmi     .done_shift
    asl
    asl
    jmp     .offset_loop
.done_shift
    iny                             ; Set y back to 0.
    ora     ($fe),y                 ; OR with the existing bytes.
    sta     ($fe),y                 ; Update the playfield.
    dex                             ; Quit after 5 rows.
    beq     .done_byte_2
    lda     $fe                     ; Update the address.
    sec
    sbc     #bytes_per_row
    bcs     .no_borrow
    dec     $ff
.no_borrow
    sta     $fe
    jmp     .byte_loop              ; Do it again!
.done_byte_2
    rts

; Erase a rock.  Arguments are as draw_rock,
; excluding the team and handle position.
erase_rock subroutine
    sty     $fd                     ; Save the offset.
    lda     #$cf                    ; Load the clearing pattern for the first row.
.shift_top_loop
    dey                             ; Rotate right until y goes negative.
    bmi     .done_shift_top
    sec
    ror
    sec
    ror
    jmp     .shift_top_loop
.done_shift_top
    pha                             ; Put this row on the stack.
    sta     $fc                     ; Save it for the last row.
    lda     #$03                    ; Load the clearing pattern for the middle rows.
    ldy     $fd                     ; Get the offset back.
.shift_middle_loop
    dey                             ; Rotate right until y goes negative.
    bmi     .done_middle_shift
    sec
    ror
    sec
    ror
    jmp     .shift_middle_loop
.done_middle_shift
    pha                             ; Push three copies of this.
    pha
    pha
    lda     $fc                     ; Get the "small" row back.
    iny                             ; Set y back to 0.
    ldx     #5                      ; Draw 5 rows.
.write_loop1
    and     ($fe),y                 ; AND with the existing bytes
    sta     ($fe),y                 ; and write back to the playfield.
    dex
    beq     .done_write1            ; Done when x hits 0.
    lda     $fe                     ; Update the address.
    clc
    adc     #bytes_per_row
    bcc     .no_carry
    inc     $ff
.no_carry
    sta     $fe
    pla                             ; Get the next row off the stack.
    jmp     .write_loop1            ; Go back and do it again.
.done_write1
    ldy     $fd                     ; Get the offset back.
    cpy     #2                      ; An offset less than 2
    bpl     .second_byte            ; requires no further action.
    rts
.second_byte
    lda     #4                      ; 4 - offset = offset for byte 2.
    sec
    sbc     $fd
    tay                             ; Put the offset in y.
    sty     $fd                     ; Save it for later.
    lda     #$cf                    ; Load the clearing pattern for the first row.
.shift_top_loop2
    dey                             ; Rotate left until y goes negative.
    bmi     .done_shift_top2
    sec
    rol
    sec
    rol
    jmp     .shift_top_loop2
.done_shift_top2
    pha                             ; Put this row on the stack.
    sta     $fc                     ; Save it for the last row.
    lda     #$03                    ; Load the clearing pattern for the middle rows.
    ldy     $fd                     ; Get the offset back.
.shift_middle_loop2
    dey                             ; Rotate left until y goes negative.
    bmi     .done_middle_shift2
    sec
    rol
    sec
    rol
    jmp     .shift_middle_loop2
.done_middle_shift2
    pha                             ; Push three copies of this.
    pha
    pha
    lda     $fc                     ; Get the "small" row back.
    inc     $fe                     ; Move one byte to the right.
    bne     .no_increment
    inc     $ff
.no_increment
    iny                             ; Set y back to 0.
    ldx     #5                      ; Draw 5 rows.
.write_loop2
    and     ($fe),y                 ; AND with the existing bytes
    sta     ($fe),y                 ; and write back to the playfield.
    dex
    beq     .done_write2            ; Done when x hits 0.
    lda     $fe                     ; Update the address.
    sec
    sbc     #bytes_per_row
    bcs     .no_borrow
    dec     $ff
.no_borrow
    sta     $fe
    pla                             ; Get the next row off the stack.
    jmp     .write_loop2            ; Go back and do it again.
.done_write2
    rts

; Draw a rock in the lower status area.  The base address of
; the rock should be in $fe, the graphics pattern should be in $fd.
draw_status_rock subroutine
    ldy     #0                      ; Index 0.
    ldx     #0                      ; Graphics index 0.
.loop
    lda     display_rock,x          ; Get the byte.
    and     $fd                     ; Mask it with the pattern.
    sta     ($fe),y                 ; Write it.
    iny
    inx
    lda     display_rock,x          ; Get the right byte.
    and     $fd                     ; Mask it.
    sta     ($fe),y                 ; Write it.
    cpx     #11                     ; Quit after 12 bytes.
    beq     .done
    jsr     advance_address
    inx
    jmp     .loop
.done
    rts

; Remove a rock in the lower status area.  The base address
; of the rock should be in $fe.
erase_status_rock subroutine
    ldy     #0                      ; Index 0.
    ldx     #5                      ; 6 rows to erase.
.loop
    lda     #0                      ; Blank out the bytes.
    sta     ($fe),y
    iny
    sta     ($fe),y
    dex
    bmi     .done
    jsr     advance_address
    jmp     .loop
.done
    rts

; Routine for advancing the address in $fe.  Tailored to
; draw_status_rock and erase_status_rock.
advance_address subroutine
    lda     $fe                     ; Get the address.
    clc
    adc     #40                     ; Go to the next row.
    bcc     .no_carry
    inc     $ff
.no_carry
    sta     $fe
    dey                             ; Reset the index.
    rts

; Draws 8 status rocks starting at the address in $fe,
; using the color pattern in $fd.
draw_8_status_rocks subroutine
    mva     $fe,$fb                 ; Save the address.
    mva     $ff,$fc
    lda     #8                      ; Store a counter, draw_status_rock
    sta     $fa                     ; clobbers all the registers.
.loop
    jsr     draw_status_rock        ; Draw the rock.
    dec     $fa
    beq     .done                   ; Quit after 8.
    lda     $fb                     ; Update the address.
    clc
    adc     #2
    bcc     .no_carry
    inc     $fc
.no_carry
    sta     $fb
    sta     $fe
    mva     $fc,$ff
    jmp     .loop                   ; Do it again!
.done
    rts

; Positions the next rock in the shooting location.
; The number of the rock should be passed in the
; accumulator.
position_next_shot subroutine
    sta     $fd                     ; Save the rock number.
    jsr     get_handedness          ; Check the shooter's handedness.
    bne     .left_handed
    mwa     throw_address_right,$fe ; Set the graphics address.
    lda     #26                     ; Set the vertical position.
    jmp     .draw_next_rock
.left_handed
    mwa     throw_address_left,$fe  ; Set the graphics address.
    lda     #39                     ; Set the vertical position.
.draw_next_rock
    ldy     $fd                     ; Get the rock number back.
    sta     rock_vertical,y
    tya
    and     #1                      ; See if the rock is odd or even.
    tax
    ldy     #0                      ; Pixel offset 0.
    lda     #6                      ; Handle position 6.
    jsr     draw_rock               ; Put the rock in front of the hack.
    rts

; Handle all logic and interactions for the rock
; whose address is in x.  Returns 0 in the accumulator
; if the rock was disabled.
process_rock subroutine
    lda     rock_h_speed,x          ; See if this rock is moving.
    bne     .rock_is_moving
    lda     rock_v_speed,x
    bne     .rock_is_moving
    jsr     get_rock_number_from_x
    jmp     .stop_rock              ; Rock is no longer moving.
.rock_is_moving
    jsr     get_rock_number_from_x  ; Grab the number for position checks.
    lda     rock_progress,y         ; Set up the collision data.
    sta     $600
    sta     $602
    lda     rock_h_speed,x          ; Check for horizontal movement.
    clc
    bmi     .check_left_move        ; Negative speed is moving left.
    bne     .check_right_move       ; Positive speed is moving right.
    jmp     .no_horizontal_move     ; Zero is no horizontal.
.check_right_move
    adc     rock_h_subpixel,x       ; Add the subpixel value.
    cmp     #100                    ; If it's over 100, try to move the rock.
    bcc     .no_horizontal_move
    inc     $600                    ; Add 1 to the desired position.
    lda     #1                      ; Save a directional indicator.
    pha
    jmp     .horizontal_done
.check_left_move
    adc     rock_h_subpixel,x       ; Add the subpixel value.
    bpl     .no_horizontal_move     ; If it's not positive, move the rock.
    lda     rock_progress,y         ; Set up the collision data.
    dec     $600                    ; Subtract 1 from the desired position.
    lda     #-1                     ; Save a directional indicator.
    pha
    jmp     .horizontal_done
.no_horizontal_move
    sta     rock_h_subpixel,x       ; Update the subpixel.
    lda     #0                      ; Save a directional indicator.
    pha
.horizontal_done
    lda     rock_vertical,y         ; Set up the collision data.
    sta     $601
    sta     $603
    lda     rock_v_speed,x          ; Check for vertical movement.
    clc
    adc     rock_v_subpixel,x
    bcs     .do_vertical_move       ; Vertical move on carry or
    cmp     #250                    ; greater than 250.
    bcs     .do_vertical_move
    jmp     .no_vertical_move
.do_vertical_move
    lda     rock_flags,x            ; Check the direction of movement.
    and     #rock_v_direction
    bne     .check_down_move
    dec     $601                    ; Subtract 1 from the desired location.
    lda     #-1                     ; Save a directional indicator.
    pha
    jmp     .vertical_done
.check_down_move
    inc     $601                    ; Add 1 to the desired location.
    lda     #1                      ; Save a directional indicator.
    pha
    jmp     .vertical_done
.no_vertical_move
    sta     rock_v_subpixel,x       ; Update the subpixel.
    lda     #0                      ; Save a directional indicator.
    pha
.vertical_done
    stx     $ff                     ; Save the rock pointer.
    lda     rock_progress_high,y    ; See if the rock has moved far enough
    bne     .distance_ok            ; for a collision check.
    lda     rock_progress,y
    cmp     #$50
    bcc     .no_collision
.distance_ok
    tsx                             ; If there are two 0s on the stack,
    lda     $101,x                  ; no collision check is needed because
    bne     .check_collision        ; the rock hasn't moved.
    lda     $102,x
    bne     .check_collision
    ldx     $ff                     ; Get the rock pointer back.
    jmp     .no_collision
.check_collision
    ldx     $ff                     ; Get the rock pointer back.
    jsr     handle_collisions
    bne     .no_collision           ; If a collision happens, no further
    pla                             ; action to be taken.  Clean up the
    pla                             ; stack and return.
    ldy     #0                      ; Reset the curl count to prevent crazy
    sty     rock_curl               ; things from happening with the new
    sty     rock_curl + 1           ; speed.
    lda     #1                      ; Rock is still alive, tell the caller.
    rts
.no_collision
    txa                             ; Save x for later use.
    pha
    lda     rock_address,x          ; Prepare the rock for erasure.
    sta     $fe
    lda     rock_address + 1,x
    sta     $ff
    ldy     rock_offset,x
    jsr     erase_rock
    pla
    tax
    jsr     get_rock_number_from_x  ; Grab the number for position updates.
    pla                             ; Check the vertical movement result.
    bmi     .move_rock_up
    bne     .move_rock_down
    jmp     .check_horizontal_move
.move_rock_up
    lda     rock_address,x          ; Adjust the address.
    sec
    sbc     #bytes_per_row
    sta     rock_address,x
    lda     rock_address + 1,x
    sbc     #0
    sta     rock_address + 1,x
    lda     rock_vertical,y
    sec
    sbc     #1
    jmp     .finish_vertical_move
.move_rock_down
    lda     rock_address,x          ; Adjust the address.
    clc
    adc     #bytes_per_row
    sta     rock_address,x
    lda     rock_address + 1,x
    adc     #0
    sta     rock_address + 1,x
    lda     rock_vertical,y
    clc
    adc     #1
.finish_vertical_move
    sta     rock_vertical,y
    lda     rock_v_subpixel,x       ; Reset the subpixel.
    clc
    adc     rock_v_speed,x
    sec
    sbc     #250
    sta     rock_v_subpixel,x
.check_horizontal_move
    pla                             ; Get the horizontal movement flag.
    bmi     .move_rock_left
    bne     .move_rock_right
    jmp     .horizontal_move_done
.move_rock_left
    lda     rock_h_subpixel,x       ; Reset the subpixel.
    clc
    adc     rock_h_speed,x
    adc     #100
    sta     rock_h_subpixel,x
    lda     rock_progress,y         ; Adjust the position.
    sec
    sbc     #1
    sta     rock_progress,y
    lda     rock_progress_high,y
    sbc     #0
    sta     rock_progress_high,y
    lda     rock_offset,x           ; Adjust the offset.
    beq     .shift_byte_left        ; On 0, the byte shifts left.
    dec     rock_offset,x
    jmp     .horizontal_move_done
.shift_byte_left
    lda     #3                      ; Reset the offset.
    sta     rock_offset,x
    lda     rock_address,x          ; Update the address.
    sec
    sbc     #1
    sta     rock_address,x
    lda     rock_address + 1,x
    sbc     #0
    sta     rock_address + 1,x
    jmp     .horizontal_move_done
.move_rock_right
    lda     rock_h_subpixel,x       ; Reset the subpixel.
    clc
    adc     rock_h_speed,x
    sec
    sbc     #100
    sta     rock_h_subpixel,x
    lda     rock_progress,y         ; Adjust the position.
    clc
    adc     #1
    sta     rock_progress,y
    lda     rock_progress_high,y
    adc     #0
    sta     rock_progress_high,y
    lda     rock_offset,x           ; Adjust the offset.
    cmp     #3
    beq     .shift_byte_right
    inc     rock_offset,x
    jmp     .horizontal_move_done
.shift_byte_right
    lda     #0                      ; Reset the offset.
    sta     rock_offset,x
    lda     rock_address,x          ; Update the address.
    clc
    adc     #1
    sta     rock_address,x
    lda     rock_address + 1,x
    adc     #0
    sta     rock_address + 1,x
.horizontal_move_done
    lda     rock_vertical,y         ; Check the upper sheet boundary.
    bpl     .check_lower
    jmp     .kill_rock
.check_lower
    cmp     #66                     ; Check the lower sheet boundary.
    bmi     .check_right
    jmp     .kill_rock
.check_right
    lda     rock_progress_high,y    ; See how far the rock has gone.
    beq     .check_foul
    lda     rock_progress,y
    cmp     #62
    bcc     .check_foul
    jmp     .kill_rock
.check_foul
    lda     shot_mode               ; If the shot mode is not "none," check
    beq     .bounds_ok              ; the hogline for a shooting foul.
    lda     rock_progress,y
    cmp     #hogline_distance
    bcc     .bounds_ok
    mva     #shot_mode_none,shot_mode
    txa                             ; Save the rock pointer, erase_curl_arrow
    pha                             ; clobbers x.
    jsr     erase_curl_arrow
    pla
    tax
    jmp     .kill_rock
.bounds_ok
    ldy     rock_r_time,x           ; Check for rotation.
    beq     .done_rotation          ; Timer at 0, not rotating.
    dec     rock_r_time,x           ; Drop the timer.
    bne     .done_rotation          ; Not yet at 0, keep waiting.
    ldy     rock_r_speed,x          ; Reset the timer.
    sty     rock_r_time,x
    lda     rock_flags,x            ; Check the rotation direction.
    and     #rock_rotation
    beq     .counterclockwise       ; 0 is counterclockwise.
    lda     rock_handle,x
    cmp     #7                      ; Wrap back to 0 after 7.
    beq     .wrap_high
    inc     rock_handle,x
    jmp     .done_rotation
.wrap_high
    lda     #0
    sta     rock_handle,x
    jmp     .done_rotation
.counterclockwise
    lda     rock_handle,x
    beq     .wrap_low               ; Wrap back to 7 after 0.
    dec     rock_handle,x
    jmp     .done_rotation
.wrap_low
    lda     #7
    sta     rock_handle,x
.done_rotation
    txa                             ; Save x for later use.
    pha
    lda     rock_address,x          ; Redraw the rock at its new location.
    sta     $fe
    lda     rock_address + 1,x
    sta     $ff
    lda     rock_flags,x            ; Dump the team to another location
    and     #rock_team              ; so we can put it in x.
    sta     $fd
    ldy     rock_offset,x
    lda     rock_handle,x
    ldx     $fd
    jsr     draw_rock
    pla                             ; Get the rock pointer back.
    tax
    jsr     get_rock_number_from_x  ; Get the rock number to index the friction.
    lda     pebble_friction         ; Get the pebble factor.
    cpy     current_rock_number     ; Apply sweeping to the shooter only.
    bne     .no_sweep_application
    sec
    sbc     sweep_value             ; Subtract the sweeping value.
.no_sweep_application
    clc
    adc     rock_friction,y
    sta     rock_friction,y
    bcs     .slow_down_rock         ; On overflow, slow the rock.
    lda     #1                      ; Rock still active.
    rts
.slow_down_rock
    lda     rock_h_speed,x
    bmi     .increase_h_speed       ; Moving left, increase speed.
    bne     .decrease_h_speed       ; Moving right, decrease speed.
    lda     rock_v_speed,x          ; Vertical move, drop speed by 5.
    sec
    sbc     #5
    bcc     .stop_rock              ; Stop the rock on underflow or zero.
    beq     .stop_rock
    sta     rock_v_speed,x          ; Update the speed.
    lda     #1                      ; Rock still active.
    rts
.decrease_h_speed
    dec     rock_h_speed,x          ; Decrease the speed.
    lda     #1                      ; Rock still active.
    rts
.increase_h_speed
    inc     rock_h_speed,x
    lda     #1                      ; Rock still active.
    rts
.kill_rock
    ldy     #0                      ; Remove the rock from the collision list.
    txa
.remove_from_active_loop
    cmp     active_rock_list,y
    beq     .found_active_rock
    iny
    jmp     .remove_from_active_loop
.found_active_rock
    lda     #$ff                    ; $ff nulls out the value.
    sta     active_rock_list,y
    jsr     get_rock_number_from_x
    lda     #0
    sta     rock_address,x
    sta     rock_address + 1,x
    sta     rock_handle,x
    sta     rock_flags,x
    sta     rock_progress,y
    sta     rock_progress_high,y
    sta     rock_vertical,y
    jmp     .zero_rock
.stop_rock
    cpy     current_rock_number     ; If this is the shooter, make a hogline check.
    bne     .hogline_ok
    lda     valid_rock_struck       ; Check to see if the shooter hit a rock
    bne     .hogline_ok             ; over the hogline, if yes, keep it.
    lda     rock_progress_high,y    ; See if the rock has crossed the hogline.
    bne     .hogline_ok
    lda     rock_progress,y
    cmp     #minimum_shot
    bcs     .hogline_ok
    lda     rock_address,x
    sta     $fe
    lda     rock_address + 1,x
    sta     $ff
    tya
    pha
    ldy     rock_offset,x
    txa
    pha
    jsr     erase_rock
    pla
    tax
    pla
    tay
    jmp     .kill_rock
.hogline_ok
    lda     #0
.zero_rock
    sta     rock_h_speed,x          ; Stop the rock.
    sta     rock_h_subpixel,x
    sta     rock_v_speed,x
    sta     rock_v_subpixel,x
    sta     rock_r_speed,x
    sta     rock_r_time,x
    jsr     deactivate_rock
    lda     #0                      ; Rock no longer active.
    rts

; Handle collision for the rock whose pointer in in x.
; The desired x position of the rock should be in $600,
; the desired y position should be in $601.
; The actual x position of the rock should be in $602,
; the actual y position of the rock should be in $603.
; Returns a 0 in the accumulator if a collision took
; place, a 1 otherwise.  Assumes the rock has moved far
; enough to consider, as only the low byte of the x
; position is compared.
handle_collisions subroutine
.mover_x                equ $600
.mover_y                equ $601
.actual_x               equ $602
.actual_y               equ $603
.target_x               equ $604
.target_y               equ $605
.mover_pointer          equ $606
.target_index           equ $607
.x_delta                equ $608
.y_delta                equ $609
.mover_h_speed          equ $60a
.mover_v_speed          equ $60b
.target_h_speed         equ $60d
.target_v_speed         equ $60e
.y4_hack_value          equ $630
    stx     .mover_pointer          ; Stash the mover pointer.
    ldy     #$ff
    sty     .target_index           ; Index into the active rock list.
.loop
    inc     .target_index           ; Go to the next rock.
    ldy     .target_index
    lda     .mover_pointer          ; Get the mover pointer.
    cmp     active_rock_list,y
    beq     .loop                   ; Don't compare a rock to itself.
    ldx     active_rock_list,y      ; Get the rock to compare against.
    bne     .not_done               ; 0 terminates the list.
    tax                             ; Restore the mover pointer.
    lda     #1                      ; Set the final result.
    rts
.not_done
    cpx     #$ff
    beq     .loop                   ; $ff is a nulled out entry.
    jsr     get_rock_number_from_x  ; Need the rock number.
    lda     rock_progress,y         ; Get the x position of the rock.
    sta     .target_x
    sec
    sbc     .mover_x                ; Subtract the mover's x position.
    cmp     #-2                     ; Less than -2, no hit.
    bmi     .loop
    cmp     #3                      ; Greater than 2, no hit.
    bpl     .loop
    sta     .x_delta                ; Save the x delta.
    lda     rock_vertical,y         ; Get the y position of the rock.
    sta     .target_y
    sec
    sbc     .mover_y                ; Subtract the mover's y position.
    cmp     #-4                     ; Less than -4, no hit.
    bmi     .loop
    cmp     #5                      ; Greater than 4, no hit.
    bpl     .loop
    sta     .y_delta
    jsr     absolute_value          ; Check for a corner no-hit.
    cmp     #4                      ; A y delta of 4 might be a corner.
    beq     .check_corner_x_0
    cmp     #3                      ; Neither 4 nor 3 is not a corner.
    bne     .not_corner
    lda     .x_delta                ; Now check the x delta.
    jsr     absolute_value
    cmp     #2                      ; An x delta of 2 is a corner no-hit.
    beq     .loop
    jmp     .not_corner
.check_corner_x_0
    lda     .x_delta                ; Now check the x delta.
    bne     .loop                   ; Only x delta 0 hits here.
.not_corner
    ldx     .mover_pointer          ; If the rock has any vertical speed,
    lda     rock_v_speed,x          ; a hack may need to be applied.
    beq     .no_y4_hack
    lda     .actual_y               ; Need to check for an edge case here.
    sec                             ; Sometimes in a y delta 4 collision
    sbc     .target_y               ; we get a case that looks like a collision,
    cmp     #4                      ; but due to the chunky pixels is not actually
    beq     .check_y4_hack_upper    ; a collision based on the movement vectors.
    cmp     #-4                     ; In this case, the collision formula will
    bne     .no_y4_hack             ; will produce nonsense results.
    lda     #-1                     ; Save a directional hack value.
    sta     .y4_hack_value
    lda     rock_flags,x            ; The rock must be moving away from
    and     #rock_v_direction       ; the target rock for the hack to apply.
    bne     .no_y4_hack
    jmp     .apply_y4_hack
.check_y4_hack_upper
    lda     #1                      ; Save a directional hack value.
    sta     .y4_hack_value
    ldx     .mover_pointer          ; The rock must be moving away from
    lda     rock_flags,x            ; the target rock for the hack to apply.
    and     #rock_v_direction
    beq     .no_y4_hack
    jmp     .apply_y4_hack
.no_y4_hack
    mwa     collision_sound,$fe     ; Collision confirmed.
    jsr     play_sound_2
    ldx     .mover_pointer          ; Get the mover's address.
    lda     rock_h_speed,x
    sta     .mover_h_speed
    lda     rock_v_speed,x
    sta     .mover_v_speed
    lda     #0
    sta     .mover_v_speed + 1      ; Treat the v_speed as 16-bit.
    lda     rock_flags,x            ; Apply a sign to the v_speed.
    and     #rock_v_direction
    bne     .mover_positive_v
    lda     #$ff
    sta     .mover_v_speed + 1
    eor     .mover_v_speed
    sta     .mover_v_speed
    inc     .mover_v_speed
    bne     .mover_positive_v
    inc     .mover_v_speed + 1
.mover_positive_v
    ldy     .target_index           ; Get the speeds of the target.
    ldx     active_rock_list,y
    lda     rock_h_speed,x
    sta     .target_h_speed
    lda     rock_v_speed,x
    sta     .target_v_speed
    lda     #0
    sta     .target_v_speed + 1
    lda     rock_flags,x
    and     #rock_v_direction
    bne     .target_positive_v
    lda     #$ff
    sta     .target_v_speed + 1
    eor     .target_v_speed
    sta     .target_v_speed
    inc     .target_v_speed
    bne     .target_positive_v
    inc     .target_v_speed + 1
.target_positive_v
    lda     .mover_pointer
    sta     .rock1_pointer
    cmp     current_rock            ; See if the shooter has hit a rock beyond
    bne     .strike_check_done      ; the hogline.
    ldy     .target_index
    lda     rock_progress_high,y
    bne     .set_strike_flag
    lda     .target_x
    cmp     #minimum_shot
    bcc     .strike_check_done
.set_strike_flag
    mva     #1,valid_rock_struck
.strike_check_done
    lda     .mover_h_speed
    sta     .rock1_velocity
    lda     .mover_v_speed
    sta     .rock1_velocity + 1
    lda     .mover_v_speed + 1
    sta     .rock1_velocity + 2
    lda     .actual_x
    sta     .rock1_position
    lda     .actual_y
    sta     .rock1_position + 1
    lda     .target_h_speed
    sta     .rock2_velocity
    lda     .target_v_speed
    sta     .rock2_velocity + 1
    lda     .target_v_speed + 1
    sta     .rock2_velocity + 2
    lda     .target_x
    sta     .rock2_position
    lda     .target_y
    sta     .rock2_position + 1
    jsr     .calculate_new_vector
    ldx     .mover_pointer          ; Reset the subpixel values.
    lda     rock_h_speed,x
    bmi     .mover_h_speed_neg
    lda     #0
    jmp     .reset_mover_v_subpixel
.mover_h_speed_neg
    lda     #99
.reset_mover_v_subpixel
    sta     rock_h_subpixel,x
    lda     #0
    sta     rock_v_subpixel,x
    ldy     .target_index
    ldx     active_rock_list,y
    stx     .rock1_pointer
    lda     .target_h_speed
    sta     .rock1_velocity
    lda     .target_v_speed
    sta     .rock1_velocity + 1
    lda     .target_v_speed + 1
    sta     .rock1_velocity + 2
    lda     .target_x
    sta     .rock1_position
    lda     .target_y
    sta     .rock1_position + 1
    lda     .mover_h_speed
    sta     .rock2_velocity
    lda     .mover_v_speed
    sta     .rock2_velocity + 1
    lda     .mover_v_speed + 1
    sta     .rock2_velocity + 2
    lda     .actual_x
    sta     .rock2_position
    lda     .actual_y
    sta     .rock2_position + 1
    jsr     activate_rock
    jsr     .calculate_new_vector
    ldx     .rock1_pointer          ; Reset the subpixel values.
    lda     rock_h_speed,x
    bmi     .target_h_speed_neg
    lda     #99
    jmp     .reset_target_v_subpixel
.target_h_speed_neg
    lda     #0
.reset_target_v_subpixel
    sta     rock_h_subpixel,x
    lda     #249
    sta     rock_v_subpixel,x
    lda     rock_v_speed,x          ; If the v speed is 0, we may need to
    bne     .no_v_speed_adjust      ; adjust it to make sure the rock moves
    lda     .position_delta + 1     ; enough to prevent a collision loop.
    beq     .no_v_speed_adjust      ; If the y position delta is 0 then 0
    bmi     .v_speed_adjust_up      ; is an acceptable y speed.
    lda     rock_flags,x
    ora     #rock_v_direction
    jmp     .set_v_speed_1
.v_speed_adjust_up
    lda     rock_flags,x
    and     #~rock_v_direction
.set_v_speed_1
    sta     rock_flags,x
    lda     #1
    sta     rock_v_speed,x
.no_v_speed_adjust
    ldx     .mover_pointer
    lda     #0                      ; Set the final result.
    ;jmp     debug_break
    rts
.calculate_new_vector
.rock1_pointer      equ $611
.rock1_velocity     equ $612
.rock1_position     equ $615
.rock2_velocity     equ $617
.rock2_position     equ $61a
.velocity_delta     equ $61c
.position_delta     equ $620
.dot_product        equ $622
.quotient           equ $624
.velocity_change    equ $625
    lda     .rock1_velocity         ; First find the velocity delta vector.
    jsr     sign_extend
    sta     $fa
    stx     $fb
    lda     .rock2_velocity
    jsr     sign_extend
    sta     $fc
    stx     $fd
    jsr     subtract_16
    lda     $fe
    sta     .velocity_delta
    lda     $ff
    sta     .velocity_delta + 1
    lda     .rock1_velocity + 1
    sta     $fa
    lda     .rock1_velocity + 2
    sta     $fb
    lda     .rock2_velocity + 1
    sta     $fc
    lda     .rock2_velocity + 2
    sta     $fd
    jsr     subtract_16
    lda     $fe
    sta     .velocity_delta + 2
    lda     $ff
    sta     .velocity_delta + 3
    lda     .rock1_position         ; Now find the position delta vector.
    sec
    sbc     .rock2_position
    sta     .position_delta
    lda     .rock1_position + 1
    sec
    sbc     .rock2_position + 1
    sta     .position_delta + 1
    lda     .velocity_delta         ; Get the dot product of the deltas.
    sta     $fa
    lda     .velocity_delta + 1
    sta     $fb
    lda     .position_delta
    sta     $fc
    jsr     signed_multiply_16_8
    lda     $fe
    sta     .dot_product            ; Stash the first value.
    lda     $ff
    sta     .dot_product + 1
    lda     .velocity_delta + 2
    sta     $fa
    lda     .velocity_delta + 3
    sta     $fb
    lda     .position_delta + 1
    sta     $fc
    jsr     signed_multiply_16_8
    lda     .dot_product
    sta     $fa
    lda     .dot_product + 1
    sta     $fb
    mva     $fe,$fc
    mva     $ff,$fd
    jsr     add_16
    lda     $fe
    sta     .dot_product
    lda     $ff
    sta     .dot_product + 1
    lda     .position_delta         ; Find the length of the position
    jsr     absolute_value          ; delta vector.
    jsr     square
    sta     $fc                     ; Save it.
    lda     .position_delta + 1
    jsr     absolute_value
    jsr     square
    clc
    adc     $fc
    sta     .quotient
    sta     $fc
    lda     .dot_product            ; Divide the dot product by the quotient.
    sta     $fa
    lda     .dot_product + 1
    sta     $fb
    jsr     signed_divide_16_8
    sty     .quotient               ; This will always be 8 bits.
    sty     $ff
    lda     .position_delta         ; Calculate the final adjustment.
    jsr     multiply
    sta     .velocity_change
    ldx     .rock1_pointer          ; Apply the change.
    lda     rock_h_speed,x
    sec
    sbc     .velocity_change
    sta     rock_h_speed,x
    lda     .position_delta + 1
    jsr     signed_multiply
    sta     .velocity_change + 1
    stx     .velocity_change + 2
    sta     $fc
    stx     $fd
    lda     .rock1_velocity + 1
    sta     $fa
    lda     .rock1_velocity + 2
    sta     $fb
    jsr     subtract_16
    lda     $ff
    bmi     .negative_v_speed
    ldx     .rock1_pointer
    lda     rock_flags,x
    ora     #rock_v_direction
    jmp     .update_flags
.negative_v_speed
    ldx     .rock1_pointer
    lda     rock_flags,x
    and     #~rock_v_direction
.update_flags
    sta     rock_flags,x
    lda     $fe
    jsr     absolute_value          ; Convert to positive.
    sta     rock_v_speed,x          ; Update the rock.
    rts
.apply_y4_hack
    ldx     .mover_pointer          ; Set up the rock to move vertically
    lda     #249                    ; on the next frame, scooting around
    sta     rock_v_subpixel,x       ; the rock it didn't actually hit.
    lda     #0                      ; Signal a collision to cancel this move.
    rts
    ;tsx                             ; Need to screw with the stack.
    ;lda     #0
    ;sta     $104,x                  ; Overwrite the projected horizontal move.
    ;lda     .y4_hack_value
    ;sta     $103,x                  ; Force a vertical move to avert the collision.
    ;ldx     .mover_pointer          ; Restore the mover pointer.
    ;lda     #1                      ; Signal no collision.
    ;rts

; Do curl calculations on the current rock.  Assumes the
; current rock pointer is in x.
apply_curl subroutine
    lda     rock_r_speed,x          ; Add the rock speed to the curl progress.
    bne     .rock_is_spinning       ; Check for "no handle."
    rts
.rock_is_spinning
    lsr                             ; Scale it down, the value stored is much
    sec                             ; larger for animation purposes.
    sbc     sweep_value             ; Subtract the sweep value.
    clc
    adc     rock_curl
    sta     rock_curl
    lda     rock_curl + 1
    adc     #0
    sta     rock_curl + 1
    cmp     #2                      ; At 512 (16-bit counter) apply curl.
    beq     .perform_curl
    rts
.perform_curl
    mva     #0,rock_curl            ; Reset the curl value.
    sta     rock_curl + 1
    lda     rock_flags,x            ; Figure out the direction of spin.
    pha                             ; Save the flags.
    and     #rock_rotation
    beq     .counterclockwise       ; 0 is counterclockwise.
    pla                             ; Get the flags back.
    and     #rock_v_direction       ; Get the current direction of motion.
    bne     .increase_speed         ; Clockwise + down.
    jmp     .decrease_speed         ; Clockwise + up.
.counterclockwise
    pla                             ; Get the flags back.
    and     #rock_v_direction       ; Get the current direction of motion.
    bne     .decrease_speed         ; Counterclockwise + down.
.increase_speed
    lda     rock_v_speed,x          ; Increase speed.
    clc
    adc     #1
    sta     rock_v_speed,x
    rts
.decrease_speed
    lda     rock_v_speed,x          ; Decrease speed.
    sec
    sbc     #1
    bcs     .same_direction         ; Direction did not change.
    lda     rock_flags,x            ; Flip the direction flag.
    eor     #rock_v_direction
    sta     rock_flags,x
    lda     #1                      ; Speed is now 1 going the other way.
.same_direction
    sta     rock_v_speed,x
    rts

; Determine if a rock is a guard.  Expects the low
; progress byte in the accumulator, the vertical in x.
; Assumes the rock progress high, byte is not 0.
; Places a 0 in the accumulator if the rock is a guard, non-zero otherwise.
rock_is_guard subroutine
    cmp     #button_left
    bcs     .not_a_guard            ; Rock is beyond the button, not a guard.
    sta     $f0                     ; Save the low byte.
    sec
    lda     #button_left            ; Find the distance.
    sbc     $f0
    jsr     rock_in_house_2
    eor     #1                      ; Invert the result of the call.
    rts
.not_a_guard
    lda     #1
    rts

; Determine if a rock is in the house.  Expects the
; rock number in y.  Assumes the rock progress high
; byte is not 0.  Places a 0 in the accumulator
; if the rock is in the house, non-zero otherwise.
; Places the horizontal distance in $f2 for the
; scoring algorithm to make use of.
rock_in_house subroutine
    lda     rock_progress,y
    cmp     #button_left            ; Figure out which side of the button
    bcc     .left_side              ; this rock is on.
    beq     .left_side
    sec
    sbc     #button_right           ; Get the horizontal distance.
    jmp     .check_distance
.left_side
    lda     #button_left            ; Get the horizontal distance.
    sec
    sbc     rock_progress,y
.check_distance
    ldx     rock_vertical,y         ; Check to see if the rock is in the house.
rock_in_house_2                     ; Alternate entry point used by rock_is_guard.
    cmp     house_edge,x
    bcc     .rock_in_house
    lda     #1                      ; Not in the house.
    rts
.rock_in_house
    sta     $f2                     ; Save the distance.
    lda     #0
    rts

