; This file is part of RealSports Curling.

; RealSports Curling is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; RealSports Curling is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with RealSports Curling.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2018 Average Software.

; Title menu routines

; Colors for the fade-in effect.
fade_in_bk      equ $20
fade_in_pf0     equ $21
fade_in_pf1     equ $22
; Timer for controlling the fade in.
fade_in_timer   equ $23

start_title subroutine
    sei                             ; Suspend interrupts.
    mwa     ENDINT,VVBLKD           ; Clear the deferred vblank.
    lda     <#null_display_list     ; Use a blank display list while
    sta     sDLISTL                 ; the title screen is being built.
    sta     DLISTL
    lda     >#null_display_list
    sta     sDLISTH
    sta     DLISTH
    mva     #4,CONSOL               ; Stick 1 keypad active.
    mva     #$8,$ff                 ; Clear the graphics RAM.
    jsr     clear_page
    mvx     #$10,$ff
.main_clear_loop
    jsr     clear_page
    cpx     #$33
    beq     .done_clear_ram
    inx
    stx     $ff
    jmp     .main_clear_loop
.done_clear_ram
    mwa     title_text_wide,$fc     ; Set up the top text.
    mwa     $800,$fe
    jsr     copy_text
    mwa     title_logo_c,$fe        ; Get the address for the big C.
    mwa     $2190,$fc               ; Target address.
    ldx     #80                     ; Row counter, there are 80 rows.
    mva     #8,$fb                  ; 8 bytes per row.
    jsr     draw_title_letter
    mwa     title_logo_u,$fe        ; Get the address for the little u.
    mwa     $2530,$fc               ; Target address.
    ldx     #52                     ; Row counter, there are 52 rows.
    mva     #7,$fb                  ; 7 bytes per row.
    jsr     draw_title_letter
    mwa     title_logo_r,$fe        ; Now the little r.
    mwa     $2537,$fc
    ldx     #51
    mva     #5,$fb
    jsr     draw_title_letter
    mwa     title_logo_l,$fe        ; Now the l.
    mwa     $2014,$fc
    ldx     #83
    mva     #5,$fb
    jsr     draw_title_letter
    mwa     title_logo_i,$fe        ; Now the i.
    mwa     $2131,$fc
    ldx     #74
    mva     #2,$fb
    jsr     draw_title_letter
    mwa     title_logo_n,$fe        ; Now the n.
    mwa     $24f3,$fc
    ldx     #52
    mva     #5,$fb
    jsr     draw_title_letter
    mwa     title_logo_g,$fe        ; Finally, the g, in two parts to
    mwa     $24f8,$fc               ; cross the 4K boundary
    ldx     #70
    mva     #8,$fb
    jsr     draw_title_letter
    mwa     title_logo_g_lower,$fe
    mwa     $3020,$fc
    ldx     #11
    mva     #8,$fb
    jsr     draw_title_letter
    mva     #0,sCOLOR0
    sta     sCOLOR1
    sta     fade_in_pf0
    sta     fade_in_pf1
    sta     sCOLOR2
    sta     sCOLBK
    sta     HPOSP0
    sta     HPOSP1
    sta     HPOSP2
    sta     HPOSP3
    sta     ATRACT
    sta     key_debounce
    mva     #$ff,last_key_pressed
    mva     #$f0,fade_in_pf0
    sta     fade_in_pf1
    mva     #1,PRIOR                ; Players over playfield.
    ldy     #$87
    jsr     build_menu_rock
    mwa     title_vblank,VVBLKD     ; Set the deferred vblank.
    mva     #$c0,NMIEN
    mva     #1,AUDCTL               ; Set the audio.
    cli                             ; Enable interrupts.
    jsr     wait_for_vb             ; Sync up before changing the display list
    mwa     title_display_list,sDLISTL
    mwa     title_display_list,DLISTL

title_loop subroutine
    mwa     title_slide_sound,$fe
    jsr     play_sound_1
    ldx     #0
.rock_slide_loop
    jsr     wait_for_vb
    inx
    inx
    cpx     #180
    beq     .start_fade_in
    stx     HPOSP0
    stx     HPOSP1
    stx     HPOSP2
    stx     HPOSP3
    jmp     .rock_slide_loop
.start_fade_in
    mva     #$0e,sCOLBK
    sta     sCOLOR2
    sta     fade_in_bk
    mva     #15,fade_in_timer
.fade_in_loop
    jsr     wait_for_vb
    dec     fade_in_timer
    bne     .fade_in_loop
    ldx     fade_in_pf0
    inx
    inx
    stx     sCOLOR0
    stx     fade_in_pf0
    ldx     fade_in_pf1
    inx
    stx     sCOLOR1
    stx     fade_in_pf1
    ldx     fade_in_bk
    dex
    dex
    stx     sCOLBK
    stx     sCOLOR2
    stx     fade_in_bk
    beq     .fade_in_done
    mva     #15,fade_in_timer
    jmp     .fade_in_loop
.fade_in_done
    mwa     title_music,$fe
    jsr     play_sound_1
    mwa     title_music_harmony,$fe
    jsr     play_sound_2
    mwa     press_start_text,$fc
    mwa     $3258,$fe
    jsr     copy_text_full
    mwa     press_license_text,$fc
    mwa     $32f8,$fe
    jsr     copy_text_full
    mwa     copyright_text,$fc
    mwa     $3320,$fe
    jsr     copy_text_full
    mva     #$ff,last_key_pressed
.loop
    jsr     wait_for_vb
    lda     last_key_pressed
    bmi     .loop
    cmp     #$0b
    beq     license_loop
    cmp     #$0c
    bne     .no_input
    jmp     start_configuration
.no_input
    mva     #$ff,last_key_pressed
    jmp     .loop

; Processing loop for the GPL display.
license_loop subroutine
    jsr     stop_all_sounds
    mwa     license_display_list,sDLISTL
    mwa     license_display_list,DLISTL,
    mwa     gpl_text_1,$fc
    mwa     $400,$fe
    jsr     copy_text_full
    mwa     gpl_text_2,$fc
    mwa     $4f0,$fe
    jsr     copy_text_full
    mwa     gpl_text_3,$fc
    mwa     $5e0,$fe
    jsr     copy_text_full
    mwa     gpl_text_4,$fc
    mwa     $6d0,$fe
    jsr     copy_text_full
    mva     #$ff,last_key_pressed
    mva     #0,HPOSP0
    sta     HPOSP1
    sta     HPOSP2
    sta     HPOSP3
.loop
    mva     #0,ATRACT
    jsr     wait_for_vb
    lda     last_key_pressed
    bmi     .loop
    cmp     #$0b
    bne     .not_done
    jmp     start_title
.not_done
    mva     #$ff,last_key_pressed
    jmp     .loop

; Draw one of the big title logo letters.  Expects:
; $fe-$ff: Address of the letter.
; $fc-$fd: Target address.
; $fb: Number of bytes per row.
; x: Number of rows.
draw_title_letter subroutine
    ldy     $fb                     ; Get the byte count.
    dey
.column_loop
    lda     ($fe),y                 ; Get the next byte.
    sta     ($fc),y                 ; Write it to RAM.
    dey
    bpl     .column_loop
    dex                             ; Check the row count.
    beq     .done
    lda     $fc                     ; Update the addresses.
    clc
    adc     #40
    sta     $fc
    lda     $fd
    adc     #0
    sta     $fd
    lda     $fe
    clc
    adc     $fb
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    jmp     draw_title_letter
.done
    rts

; Deferred vblank hander.
title_vblank subroutine
    jsr     process_sounds
    ldy     key_debounce            ; Process the debounce timer.
    beq     .no_debounce
    dec     key_debounce
.no_debounce
    jmp     ENDINT

